/*
  	#[ Include file :
*/
#include "stedi.h"
/*
  	#] Include file : 
  	#[ Print routines :
 		#[ Print buffer :

		This function is meant for the Atari ST only. It avoids the
		problem with GEMDOS not handing over the errorcodes of the
		BIOS. This could cause all kinds of nasty delays when printing
		large files with the printer not ready.

 		#] Print buffer : 
 		#[ Print Command :
*/

LONG
Print(UBYTE *s,WORD nn)
{
	BUFFER b = cbuf;
	WORD x, y, n = nn;
	DEFAULT d = &dbufFer;
	LONG ret;
	if ( n <= 0 ) {			/* Normal print command. */
		x = b->outputmode;
		b->outputmode = PRINTMODE;
	}
	else if ( *s == 'f' || *s == 'F' ) { /* Print with pagenumbers */
		s++; n--;
		SKIPBLANKS(s,n);
		if ( n > 0 ) return(Syntax());
		x = b->outputmode;
		b->outputmode = FORMATMODE;
	}
	else if ( *s == 'p' || *s == 'P' ) { /* Set or read page or offset */
		s++; n--;
		SKIPBLANKS(s,n);
		if ( n > 0 ) {					/* set new pagelayout */
			s = ReadNum(s,&n,&x);
			if ( x < 6 ) x = 6;
			x -= 3;
			SKIPBLANKS(s,n);
			if ( n > 0 ) {
				if ( *s != ',' ) return(Syntax());
				s++; n--;
				SKIPBLANKS(s,n);
				s = ReadNum(s,&n,&y);
				SKIPBLANKS(s,n);
			}
			if ( n > 0 ) return(Syntax());
			d->maxinpage = x;
			d->printoffset = y;
		}
				/* read page settings */
		MesPrint(5,d->maxinpage+3,d->printoffset);
		/* "%d lines per formatted printerpage. Offset = %d" */
		return(0L);
	}
	else if ( *s == '=' ) {
		n--; s++;
		SKIPBLANKS(s,n);
		if ( n > 0 ) {
			if ( n < 4 ) return(Syntax());
			if ( strNIcmp(s,(UBYTE *)("PRN:"),4) == 0 ) {
				s += 4; n -= 4; SKIPBLANKS(s,n);
				if ( n > 0 ) return(Syntax());
				Strcpy((UBYTE *)("PRN:"),(UBYTE *)(d->printer));
			}
			else if ( strNIcmp(s,(UBYTE *)("AUX:"),4) == 0 ) {
				s += 4; n -= 4; SKIPBLANKS(s,n);
				if ( n > 0 ) return(Syntax());
				Strcpy((UBYTE *)("AUX:"),(UBYTE *)(d->printer));
			}
			else return(Syntax());
		}
		MesPrint(6,d->printer); /* "Printing to %s" */
		return(0L);
	}
	else return(Syntax());
	AppendFlag = 0;
	ret = WriteFile((UBYTE *)(d->printer),1);
	b->outputmode = x;
	if ( ret >= 0 ) {
		MesPrint(7,ret); /* "%l lines sent to the printer." */
		ret = 0;
	}
	else if ( ret == -3 ) {
		MesnOut(18); /* "Printer not available" */
	}
	return(ret);
}

/*
 		#] Print Command : 
  	#] Print routines : 
  	#[ Save routines :
 		#[ klaar :
*/

LONG
klaar(VOID)
{
	BUFFER b;
	WORD i, haveone = 0;
	UBYTE ss[12], *s;
	if ( BlockExit ) return(0);
	s = ss;
	for ( i = 0; i < 8; i++ ) {
		b = textbuffer[i];
		if ( b->dirtyflag > 0 ) {
			*s++ = ((i+1)%10) + '0';
			haveone++;
		}
	}
	if ( haveone ) {
		*s = 0;
		MesPrint(8,ss);
		if ( MesAsk(98,INCOMMAND) <= 0 ) return(0);
		/* "Unsaved buffer(s) %s Leave anyway? (Y/N) " */
	}
	return(EXITCODE);
}

/*
 		#] klaar : 
 		#[ Forced save :
*/

LONG
save(VOID)
{
	LONG ret;
	if ( BlockExit ) return(0);
	if ( cbuf->backupmode == VIEWONLY ) {
		MesnOut(21); /* "Buffer is ViewOnly." */
		return(-1L);
	}
	if ( *(cbuf->fullname) == 0 ) {
		MesnOut(22); /* "Cannot save a buffer without a name" */
		return(-1);
	}
	AppendFlag = 0;
	ret = WriteFile(cbuf->fullname,0);
	if ( ret < 0 ) return(ret);
	cbuf->dirtyflag = 0;
	MesPrint(4,cbuf->totline); /* "%l lines written." */
	return(0L);
}

/*
 		#] Forced save : 
 		#[ Save + Quit :
*/

LONG
savek(VOID)
{
	LONG ret;
	if ( cbuf->dirtyflag ) { if ( ( ret = save() ) < 0 ) return(ret); }
	return(klaar());
}

/*
 		#] Save + Quit : 
 		#[ The Save command :
*/

LONG
SaveS(UBYTE *s,WORD n)
{
	if ( n <= 0 ) return(save());
	if ( *s == 'q' || *s == 'Q' ) {
		n--; s++;
		SKIPBLANKS(s,n);
		if ( n <= 0 ) return(savek());
	}
	return(Syntax());
}

/*
 		#] The Save command : 
 		#[ The Quit command :
*/

LONG
Quit(UBYTE *s,WORD n)
{
	if ( n > 0 ) return(Syntax());
	return(klaar());
}

/*
 		#] The Quit command : 
  	#] Save routines : 
  	#[ Error messages :
*/

BYTE *ErrMessages[] = {
	 " File not found."
	," Path not found."
	," No more handles."
	," Access denied."
	," No memory."
	," Drive not found."
	," Invalid load format."
};

LONG
ErrorMes(LONG number,WORD par)
{
	WORD num;
	num = (WORD)(-number);
	if ( par == 0 ) {
		MesnOut(23); /* "" */
	}
	if ( num <= 36 && num >= 33 ) num -= 33;
	else if ( num == 39 ) num = 4;
	else if ( num == 46 ) num = 5;
	else if ( num == 66 ) num = 6;
	else {
			if ( number < 0 ) { MesCon(" Error "); MesNum(number); }
			else if ( number > 0 ) { MesCon("Return code = "); MesNum(number); }
			return(number);
	}
	MesCon(ErrMessages[num]);
	return(number);
}
 
VOID
MesMem(VOID)
{
	MesOut(ErrMessages[4]);
}

/*
  	#] Error messages : 
  	#[ Defaults :
 		#[ From file to defaults buffer :		FilToDef()
*/

#ifdef ANSI
#include <string.h>
#endif

UBYTE defaultdir[MAXIPATH];

LONG
FilToDef(WORD par)
{
	struct defbuf dd;
	DEFAULT d = &dbufFer;
	UBYTE *s;
	WORD handle, oldcolor;
	LONG filelength;
	oldcolor = d->color;
	if ( ( handle = (WORD)(MyOpen((UBYTE *)DefName,O_RDONLY)) ) < 0 ) { 
		s = defaultdir;
		while ( *s ) s++;
		Strcpy((UBYTE *)DefName,s);
		if ( ( handle = (WORD)(MyOpen(defaultdir,O_RDONLY)) ) < 0 ) {
			if ( !par ) {
				MesPrint(9,defaultdir); /* "Cannot find file %s" */
			}
			*s = 0;
			return(-1L);
		}
		*s = 0;
	}
	if ( ( filelength = FileSeek(0L,(WORD)handle,2) ) < sizeof(struct defbuf) ) {
		if ( !par ) {
			MesnOut(24); /* "Improper defaults file." */
		}
		CloseFile((WORD)handle);
		return(-1L);
	}
	FileSeek(0L,(WORD)handle,0);
	if ( FileRead(handle,(LONG)sizeof(struct defbuf),(BYTE *)(&dd))
	!= sizeof(struct defbuf) || dd.version != DEFVERSION ) {
		CloseFile(handle);
		if ( dd.version != DEFVERSION ) {
			if ( !par ) {
				MesnOut(25); /* "Wrong version default file." */
			}
			else {
				Cconws("Invalid default file.");
WeGoOut:
				Cconws(" Hit any key to abort\r\n");
				Bconin(2);
				exit(-1);
			}
		}
		else {
ReadErr:
			MesnOut(26); /* "Error while reading." */
		}
		return(-1L);
	}
	*d = dd;
	if ( FileRead(handle,(LONG)(MAXLEARN*10*sizeof(WORD)),(BYTE *)LearnBuffer)
	!= (MAXLEARN*10*sizeof(WORD)) ) goto ReadErr;

	if ( par == 0 ) {
		if ( ( ( d->color ^ oldcolor ) & 1 ) != 0 ) PScolor();
		return(0L);
	}
/*
 *	Here we come only at startup!
 */
	if ( d->redeflength > 0 ) {
		KeyPointer = (LONG *)(MALLOC(d->redeflength));
		if ( KeyPointer == (LONG *)0 ) {
			NumberKeys = d->redeflength = d->numredef = 0;
NoMem:		CloseFile(handle);
			return(NOMEMORYL);
		}
		if ( FileRead(handle,d->redeflength,(BYTE *)KeyPointer) < d->redeflength ) {
			Cconws("Error while reading keyredefinitions.");
			goto WeGoOut;
/*
			Mfree(KeyPointer);
			NumberKeys = d->redeflength = d->numredef = 0;
			CloseFile(handle);
			return(-1L);
*/
		}
		NumberKeys = d->numredef;
	}
	if ( d->macros ) {
		MACRO m;
		LINE mm;
		filelength -= sizeof(struct defbuf) + d->redeflength
			+(MAXLEARN*10*sizeof(WORD));
		if ( ( FirstMacro = (MACRO)MALLOC(filelength) ) == 0 ) goto NoMem;
		if ( FileRead(handle,filelength,(BYTE *)FirstMacro) < filelength ) {
			Cconws("Error while reading macros.");
			goto WeGoOut;
/*
			Mfree((BYTE *)FirstMacro);
			FirstMacro = 0;
			CloseFile(handle);
			return(-1L);
*/
		}
		m = (MACRO)(&FirstMacro);
		while ( m->next ) {
			m = m->next;
			filelength -= m->size;
			m->next = (MACRO)(((BYTE *)m) + m->size);
			m->ownmalloc = 0;
			m->used = 0;
			mm = (LINE)(((BYTE *)m) + m->start);
			if ( mm < (LINE)(m->next) ) {
				for (;;) {
					mm->prev = 0;
					mm->next = (LINE)(((BYTE *)mm) + mm->totlen);
					if ( mm->next >= (LINE)(m->next) ) { mm->next = 0; break; }
					mm = mm->next;
				}
			}
			DoAll(m);
			if ( filelength <= 0 ) m->next = 0;
		}
	}
	CloseFile(handle);
	return(0L);
}

/*
 		#] From file to defaults buffer : 
 		#[ From defaults buffer to file :		DefToFil()
*/

LONG
DefToFil(VOID)
{
	UBYTE name[MAXIPATH+10];
	DEFAULT d = &dbufFer;
	UBYTE *t, *s;
	WORD handle;
	if ( FirstMacro ) d->macros = 1;
	else d->macros = 0;
	s = defaultdir;
	t = name;
	while ( *s ) *t++ = *s++;
	Strcpy((UBYTE *)DefName,t);
	if ( ( handle = (WORD)(MyCreate(name,0)) ) < 0 ) {
		MesPrint(10,name); /* "Cannot create file %s" */
		return(-1L);
	}
	if ( FileWrite(handle,(LONG)sizeof(struct defbuf),(BYTE *)d)
			!= sizeof(struct defbuf) ) goto ErrorM;
	if ( FileWrite(handle,(LONG)(MAXLEARN*10*sizeof(WORD)),(BYTE *)LearnBuffer)
			!= (LONG)(MAXLEARN*10*sizeof(WORD)) ) goto ErrorM;
	if ( d->redeflength > 0 ) {
		if ( FileWrite(handle,d->redeflength,(BYTE *)KeyPointer)
					< d->redeflength ) {
ErrorM:
			MesnOut(27); /* "Error while writing. File unreliable." */
			CloseFile(handle);
			return(-1L);
		}
	}
	if ( d->macros ) {
		MACRO m;
		m = (MACRO)(&FirstMacro);
		while ( m->next ) {
			m = m->next;
			if ( FileWrite(handle,m->size,(BYTE *)m) != m->size ) {
				goto ErrorM;
			}
		}
	}
	CloseFile(handle);
	MesPrint(11,name); /* "Defaults written to %s" */
	return(0L);
}

/*
 		#] From defaults buffer to file : 
 		#[ From textbuffer to defaults buffer : BufToDef()
*/

VOID
BufToDef(VOID)
{
	BUFFER b = cbuf;
	DEFAULT d = &dbufFer;
	WORD i;
	UBYTE *s, *t;
	d->iomode		= b->iomode;
	d->outputmode	= b->outputmode;
	d->direction	= b->direction;
	d->sensitivity	= b->sensitivity;
	if ( b->backupmode != VIEWONLY ) d->backupmode = b->backupmode;
	d->wordwrap		= b->wordwrap;
	d->autoindent	= b->autoindent;
	s = b->tabstring; t = d->tabstring;
	i = 100;
	do { *t++ = *s++; } while ( --i > 0);
}

/*
 		#] From textbuffer to defaults buffer : 
 		#[ From defaults buffer to textbuffer : DefToBuf()
*/

VOID
DefToBuf(VOID)
{
	BUFFER b = cbuf;
	DEFAULT d = &dbufFer;
	WORD i;
	UBYTE *s, *t;
	b->iomode		= d->iomode;
	b->outputmode	= d->outputmode;
	b->direction	= d->direction;
	b->sensitivity	= d->sensitivity;
	b->backupmode	= d->backupmode;
	b->wordwrap		= d->wordwrap;
	b->autoindent	= d->autoindent;
	s = b->tabstring; t = d->tabstring;
	i = 100;
	do { *s++ = *t++; } while ( --i > 0);
}

/*
 		#] From defaults buffer to textbuffer : 
 		#[ Show the defaults directory : 		ShowDef()
*/

VOID
ShowDef(VOID)
{
	MesPrint(12,defaultdir); /* "DFT file from %s" */
}

/*
 		#] Show the defaults directory : 
 		#[ Set a new defaults directory :		SetDef()
*/

WORD InStart = 0;

VOID
SetDef(UBYTE *s,WORD n)
{
	UBYTE *t;
	t = defaultdir;
	if ( n > 0 ) {
		if ( n > (MAXIPATH-2) ) n = (MAXIPATH-2);
		while ( --n >= 0 ) *t++ = *s++;
		if (
#ifdef WITHALTSEPARATOR
		 ( *(t-1) != ALTSEPARATOR ) &&
#endif
		 ( *(t-1) != SEPARATOR ) ) *t++ = SEPARATOR;
	}
	*t++ = '\0';
#ifdef WITHALTSEPARATOR
	t = defaultdir;
	while ( *t && ( *t != SEPARATOR ) && ( *t != ALTSEPARATOR ) ) t++;
	if ( *t == SEPARATOR ) {
		while ( *t ) {
			if ( *t == ALTSEPARATOR ) *t = SEPARATOR;
			t++;
		}
	}
	else {
		while ( *t ) {
			if ( *t == SEPARATOR ) *t = ALTSEPARATOR;
			t++;
		}
	}
#endif
	if ( !InStart ) ShowDef();
}

VOID
setdefdir(UBYTE *s)
{
	WORD n = 0;
	UBYTE *t;
	t = s;
	while ( *t ) { t++; n++; }
	InStart = 1;
	SetDef(s,n);
	InStart = 0;
}

/*
 		#] Set a new defaults directory : 
 		#[ Default commands :
*/

LONG
Dfaults(UBYTE *s,WORD n)
{
	LONG ret;
	WORD c;
	if ( n <= 0 ) return(Syntax());
	n--;
	c = ToUpper[*s++];
	if ( c == 'D' ) {
		if ( n > 0 ) {
			if ( *s == '\'' || *s == '=' || *s == ' ' || *s == '\t' ) {
				n--; s++;
			}
			else return(Syntax());
		}
		SKIPBLANKS(s,n);
		if ( n <= 0 ) ShowDef();
		else          SetDef(s,n);
		return(0L);
	}
	SKIPBLANKS(s,n);
	if ( n <= 0 ) {
		switch ( c ) {
			case 'W' :
				BufToDef();
			case 'E' :
				return(DefToFil());
			case 'R' :
				if ( !LearnReplay && !learning ) {
					if ( ( ret = FilToDef(0) ) < 0 ) return(ret);
					DefToBuf();
					return(0L);
				}
			case 'I' :
				if ( !LearnReplay && !learning ) return(FilToDef(0));
				MesnOut(28); /* "Cannot read defaults from learnbuffer." */
				return(-1L);
			case 'L' :
				DefToBuf();
				return(0L);
			case 'S' :
				BufToDef();
				return(0L);
			default :
				break;
		}
	}
	return(Syntax());
}

/*
 		#] Default commands : 
  	#] Defaults : 
  	#[ Help File :
*/

LONG
help(VOID)
{
	BUFFER b;
	UBYTE *t;
	WORD i;
	LONG ret;
	if ( oldbuffer >= 0 ) {
		return(GotoBuffer(oldbuffer));
	}
	i = 7;
	do {
		b = textbuffer[i];
		if ( ( b->current == (LINE)b ) || ( strcmp1((UBYTE *)HlpName,b->name) == 0 ) ) {
			if ( b->dirtyflag == 0 && b->current != (LINE)b ) {
				i = cbuf->bufnum;
				GotoBuffer(b->bufnum);
				oldbuffer = i;
				return(0);
			}
			BufUnd(i);
			ClearUndo();
			inibuf(i);
			break;
		}
	} while ( --i >= 0 );
	if ( i < 0 ) {
		MesnOut(29); /* "No slot for" */
		goto AndEnd;
	}
	t = (UBYTE *)HlpName;
	if ( ( ret = MyOpen(t,O_RDONLY) ) < 0 && ( !HelpName || 
	     ( ret = MyOpen((t = HelpName),O_RDONLY) ) < 0 ) ) {
		MesnOut(30); /* "Cannot find" */
AndEnd:
		MesCon(" help file. Quitting is with shift-F10.");
		return(-1L);
	}
	CloseFile((WORD)ret);
	oldbuffer = cbuf->bufnum;
	cbuf = b;
	b->backupmode = VIEWONLY;
	StartPos = 0;
	ret = ReadFile(t);
	if ( ret >= 0 ) ForceDisplay();
	else {
		cbuf = textbuffer[oldbuffer];
		oldbuffer = -1;
		GotoBuffer(cbuf->bufnum);
	}
	return(ret);
}

/*
  	#] Help File : 
*/

