/*
  	#[ Include file :
*/
#include "stedi.h"
/*
  	#] Include file : 
  	#[ Memory routines :
 		#[ Startup of memory : WORD StartMem()

		This is the allocation of the first memory block.
		The first block must always be there. That simplifies
		all kinds of operations, because they don't have to worry
		about exceptional cases.
*/

MBLOCK FirstMem, LastMem, CurrentMem, vMem;
LINE   TableRMO[24+NUMTAGS];
LINE   *Table2RMO[24+NUMTAGS];
WORD   numRMO;

WORD
StartMem(VOID)
{
	MBLOCK m;
	if ( ( m = (MBLOCK)MALLOC((LONG)MEMLENGTH) ) == 0 ) {
		MesMem();
		return(-1);
	}
	vMem = CurrentMem = LastMem = FirstMem = m;
	m->next = m->prev = (MBLOCK)0;
	m->user = m->fill = (LINE)(m->cdata);
	m->SpaceAtEnd = m->AllSpace = (WORD)(MEMLENGTH-MHEADLENGTH);
	return(0);
}

/*
 		#] Startup of memory : 
 		#[ Line allocation : LINE LineAlloc(num,after)

		Allocate the space for a single line. Note that we work
		in units of 16 bytes.
		Under no circumstances may this routine execute a garbage
		collection if there is not enough memory. The calling
		routines can have all kinds of pointers floating around.
*/

LINE
LineAlloc(WORD num,LINE after)
{
	WORD n;
	LINE l;
	MBLOCK m = LastMem;
	n = ( num + (LINESIZE-1) + LINEHEAD ) & ( -(LINESIZE) );
	if ( m->SpaceAtEnd < n ) {	/* Block too full. Get a new one. */
		if ( ( m = (MBLOCK)MALLOC((LONG)MEMLENGTH) ) == 0 ) {
			MesMem();
			return((LINE)0);
		}
		LastMem->next = m;
		m->prev = LastMem;
		LastMem = m;
		m->next = (MBLOCK)0;
		m->user = m->fill = (LINE)(m->cdata);
		m->SpaceAtEnd = m->AllSpace = (WORD)(MEMLENGTH-MHEADLENGTH);
		if ( m->SpaceAtEnd < n ) {
			MesnOut(14); /* "Line too long." */
			return((LINE)0);
		}
	}
	m->SpaceAtEnd -= n;
	m->AllSpace   -= n;
	l = m->fill;
#ifdef FASTCOPY
	(UBYTE *)(m->fill) += n;
#else
	m->fill = (LINE)(((UBYTE *)m->fill) + n);
#endif
	l->next = after->next;		/* Put the links in */
	after->next = l;
	l->next->prev = l;
	l->prev = after;
	l->totlen = n;				/* Total length */
	l->used = num + LINEHEAD;	/* Length of the line */
	return(l);
}

/*
 		#] Line allocation : 
 		#[ Foldline allocation : FOLDLINE FoldAlloc(num,after)
*/

FOLDLINE
FoldAlloc(WORD num,LINE after)
{
	FOLDLINE newline;
	newline = (FOLDLINE)LineAlloc(num+FOLDHEAD-LINEHEAD,after);
	if ( newline == (FOLDLINE)0 ) return((FOLDLINE)0);
	newline->first = newline->last = (LINE)(((UBYTE *)newline) + LINEHEAD);
	newline->lines = 0;
	newline->used = newline->mark;
	newline->mark = FOLDCODE;
	return(newline);
}

/*
 		#] Foldline allocation : 
 		#[ Line deallocation : VOID LineDealloc(l)

		This should be changed somewhat.
		If a whole buffer is emptied (undo with more than ?? lines)
		we should not update the space.
		After the returning there should then be a special garbage collection
		which doesn't look at AllSpace.
*/

VOID
LineDealloc(LINE l)
{
	MBLOCK m, mm;
#ifdef DEBUGON
	if ( l == (LINE)cbuf ) {
		MesOut("Attempt to deallocate cbuf");
		KeyFlush();
		KeyGet();
	}
#endif
	m = mm = vMem;
	do {
		if ( ( P_LO(l) > P_LO(m) ) && ( P_LO(l) < ( MEMLENGTH + P_LO(m) ) ) ) {
			m->AllSpace += l->totlen;
			vMem = m;
			l->next->prev = l->prev;	/* Take out from the link */
			l->prev->next = l->next;
			l->next = (LINE) 0;			/* Mark as garbage */
			return;
		}
		m = m->next;
		if ( m == 0 ) m = FirstMem;
	} while ( m != mm );
	MesnOut(34); /* "Deallocation error" */
}

/*
 		#] Line deallocation : 
 		#[ Drop part of a line : VOID DropPart(l,num)
*/

VOID
DropPart(LINE l,WORD num)
{
	WORD i;
	MBLOCK m = LastMem;
#ifdef DEBUGON
	if ( l == (LINE)cbuf || l == (LINE)undoB ) {
		MesOut("Attempt to drop part of cbuf or undoB");
		KeyFlush();
		KeyGet();
	}
#endif
	l->used -= num;
	if ( l->used < LINEHEAD ) {
/*
		Some debug code
*/
#ifdef DEBUGON
		MesOut("Error in DropPart");
		KeyFlush();
		KeyGet();
#endif
		l->used = LINEHEAD;
	}
	i = l->used;
	i = ( i + (LINESIZE-1) ) & (-(LINESIZE));
	if ( i < l->totlen ) {
		for (;;) {
			if ( ( P_LO(l) > P_LO(m) ) && ( P_LO(l) < ( MEMLENGTH + P_LO(m) ) ) ) {
				num = l->totlen - i;
				l->totlen = i;
				l = (LINE)(((BYTE *)l) + i);
				l->totlen = num;
				l->next = (LINE)0;			/* Mark as garbage */
				m->AllSpace += num;
				return;
			}
			else {
				if ( ( m = m->prev ) == 0 ) {
					if ( InMustWrite > 0 ) return;
					MesnOut(34); /* "Deallocation error" */
					return;
				}
			}
		}
	}
}

/*
 		#] Drop part of a line : 
 		#[ Garbage collection : WORD Garbage()
			#[ Comments :

		The routines here are somewhat different from the ones in the
		assembler version. First the RMO addresses are stored together
		with the address where this RMO is kept. This costs more space
		but it is gained back by the speed with which the new link can
		be made. Second the whole routine will give off a code if no
		task was performed. This allows a direct waiting for an event
		in a multitasking environment. The assembler version keeps using
		time, even when nothing needs to be done anymore. The assembler
		version may get changed sooner or later.

			#] Comments : 
			#[ Full collection :
*/

LONG
garb(VOID)
{
	while ( Garbage() ) {}
	return(0);
}

/*
			#] Full collection : 
			#[ Find Task :
*/

WORD
Garbage(VOID)
{
	MBLOCK m;
	WORD result;
	m = CurrentMem;
	do {
		if ( m->SpaceAtEnd != m->AllSpace ) { CleanBlock(m); return(1); }
		if ( ( m != LastMem ) && ( result = FillBlock(m) ) != 0 ) return(result);
		if ( ( m = m->next ) == 0 ) m = FirstMem;
	} while ( m != CurrentMem );
	return(0);
}

/*
			#] Find Task : 
			#[ Clean block :
*/

VOID
CleanBlock(MBLOCK m)
{
	LINE l1, l2;
	WORD len2;
	MakeRMO(m);
	l2 = l1 = m->user;		/* l1 is for filling, l2 finds lines. */
	while ( l2 < m->fill ) {
		len2 = l2->totlen;
		if ( l2->next ) {
			if ( l1 != l2 ) MoveLine(l2,l1);
#ifdef FASTCOPY
			(UBYTE *)l1 += l1->totlen;
		}
		(UBYTE *)l2 += len2;
#else
			l1 = (LINE)(((UBYTE *)l1) + l1->totlen);
		}
		l2 = (LINE)(((UBYTE *)l2) + len2);
#endif
	}
	m->fill = l1;
	m->SpaceAtEnd = m->AllSpace;
	if ( m->AllSpace == DATALENGTH && m == LastMem && m != FirstMem ) {
		l2 = (LINE)(LastMem->prev);
		if ( CurrentMem == LastMem ) CurrentMem = (MBLOCK)l2;
		if ( vMem == LastMem ) vMem = (MBLOCK)l2;
		Mfree(LastMem);
		LastMem = (MBLOCK)l2;
		LastMem->next = 0;
	}
}

/*
			#] Clean block : 
			#[ Fill block :
*/

WORD
FillBlock(MBLOCK m)
{
	LINE l1;
	LINE l2;	/* l1 runs in m, l2 in LastMem */
	MBLOCK mlast = LastMem;
	while ( mlast->fill == mlast->user
	|| mlast->AllSpace >= DATALENGTH ) {	/* Last block is empty */
		l2 = (LINE)(mlast->prev);
		if ( CurrentMem == mlast ) CurrentMem = (MBLOCK)l2;
		if ( vMem == mlast ) vMem = (MBLOCK)l2;
		Mfree(mlast);
		LastMem = mlast = (MBLOCK)l2;
		mlast->next = 0;
		if ( m == mlast ) return(1);
	}
	l1 = m->fill;
	l2 = mlast->user;
#ifdef FASTCOPY
	while ( !(l2->next) ) (UBYTE *)l2 += l2->totlen;
#else
	while ( !(l2->next) ) l2 = (LINE)(((UBYTE *)l2) + l2->totlen);
#endif
	if ( l2->totlen > m->SpaceAtEnd ) return(0);
	MakeRMO(mlast);
	do {
		m->SpaceAtEnd -= l2->totlen;
		m->AllSpace   -= l2->totlen;
		MoveLine(l2,l1);
#ifdef FASTCOPY
		(UBYTE *)l1 += l1->totlen;
#else
		l1 = (LINE)(((UBYTE *)l1) + l1->totlen);
#endif
		l2->next = 0;
		mlast->AllSpace += l2->totlen;
#ifdef FASTCOPY
		(UBYTE *)l2 += l2->totlen;
#else
		l2 = (LINE)(((UBYTE *)l2) + l2->totlen);
#endif
		if ( mlast->AllSpace >= DATALENGTH ) {	/* Block is empty now */
			l2 = (LINE)(mlast->prev);
			if ( CurrentMem == mlast ) CurrentMem = (MBLOCK)l2;
			if ( vMem == mlast ) vMem = (MBLOCK)l2;
			Mfree(mlast);
			LastMem = mlast = (MBLOCK)l2;
			mlast->next = 0;
			m->fill = l1;
			return(1);
		}
#ifdef FASTCOPY
		while ( !(l2->next) ) (UBYTE *)l2 += l2->totlen;
#else
		while ( !(l2->next) ) l2 = (LINE)(((UBYTE *)l2) + l2->totlen);
#endif
	} while ( l2->totlen <= m->SpaceAtEnd );
	m->fill = l1;
	return(1);
}

/*
			#] Fill block : 
			#[ Move line :

		When a line gets moved all pointers to that line should be
		updated. These can all be found in the RMO table.
*/

VOID
MoveLine(LINE a,LINE b)
{
	WORD i;
	LINE *t;
	a->next->prev = b;
	a->prev->next = b;
	if ( a->used < 0 ) {		/* Foldline */
		((FOLDLINE)a)->first->prev = ((FOLDLINE)a)->last->next =
			(LINE)(&(((FOLDLINE)b)->first));
	}
	t = TableRMO;
	for ( i = 0; i < numRMO; i++ ) {
		if ( a == *t++ ) *(Table2RMO[i]) = b;
	}
#ifdef SLOWCOPY
	i = a->totlen;
	while ( --i >= 0 ) *b++ = *a++;
#else
/*	i = a->totlen >> 4; */
	i = a->totlen / (4*sizeof(LONG));
#ifdef FASTCOPY
	do {
		   *((LONG *)b)++ = *((LONG *)a)++;
		   *((LONG *)b)++ = *((LONG *)a)++;
		   *((LONG *)b)++ = *((LONG *)a)++;
		   *((LONG *)b)++ = *((LONG *)a)++;
	} while ( --i > 0 );
#else
	{
		LONG *from, *to;
		from = (LONG *)a;
		to = (LONG *)b;
		do {
			*to++ = *from++;
			*to++ = *from++;
			*to++ = *from++;
			*to++ = *from++;
		} while ( --i > 0 );
	}
#endif
#endif
}

/*
			#] Move line : 
			#[ Make RMO table :

		RMO stands for Relevant Movable Objects.
		When lines get moved, there may be pointers to such lines.
		The editor has been programmed such that the number of pointers
		to lines, that are not part of the links that can be reconstructed
		from information in the line, is. very limited. They are the
		current line pointers and the mark pointers for each of the 12
		buffers (8 text buffers, two yank buffers, the command line and
		the undo buffer). In addition there are the tags. We collect the
        addresses where these line pointers are stored. For speed we also
        keep the line pointers themselves.

		The bussiness with the RMO pointers can be sped up very much
		by sorting them. Still to be done.
*/

VOID
MakeRMO(MBLOCK mm)
{
	BUFFER b;
	ULONG l, m, mtop;
	WORD i = 11;	/* Note that this must be 12-1 */
	LINE tline;
	m = P_LO(mm);
	mtop = m + MEMLENGTH;
	numRMO = 0;
	do {
		b = textbuffer[i];
		l = P_LO(b->current);
		if ( ( l > m ) && ( l < mtop ) ) {
			Table2RMO[numRMO] = &(b->current);
			TableRMO[numRMO++] = b->current;
		}
		l = P_LO(b->mark);
		if ( ( l > m ) && ( l < mtop ) ) {
			Table2RMO[numRMO] = &(b->mark);
			TableRMO[numRMO++] = b->mark;
		}
	} while ( --i >= 0 );
	i = NUMTAGS-1;
	do {
		tline = tags[i].line;
		l = P_LO(tline);
		if ( ( l > m ) && ( l < mtop ) ) {
			Table2RMO[numRMO] = &(tags[i].line);
			TableRMO[numRMO++] = tline;
		}
	} while ( --i >= 0 );
}

/*
			#] Make RMO table : 
 		#] Garbage collection : 
  	#] Memory routines : 
  	#[ Line utilities :
 		#[ Column -> Character :	WORD ColToChar(line,col)

		Simple conversion of a column to the number of a character which
		counts from zero for the character. The answer is -1 if the
		column is beyond the last character.
*/

WORD
ColToChar(LINE line,WORD col)
{
	UBYTE *tabstring = cbuf->tabstring;
	UBYTE *t;
	WORD n, i, c;
	if ( line->used < 0 ) line = (LINE)(line->text);
	t = line->text;
	n = line->used - LINEHEAD;
	i = 0; c = 1;
	while ( --n >= 0 ) {
		if ( *t == '\t' && *tabstring ) {
			while ( *tabstring && *tabstring < (c-1) ) tabstring++;
			if ( *tabstring ) c = *tabstring++ + 1;
		}
		if ( c >= col ) return(i);
		c++; i++; t++;
	}
	return(-1);
}

/*
 		#] Column -> Character : 
 		#[ Column -> Character :	WORD FindChar(l,col,pos)
*/

WORD
FindChar(LINE line,WORD column,FINDPOS *position)
{
	WORD num = 0;
	UBYTE *tabstring;
	WORD nchar;
	UBYTE *instring;
	UBYTE zero = 0;
	if ( ( nchar = line->used ) < 0 ) {
		nchar = ((FOLDLINE)line)->used - FOLDHEAD;
		instring = ((FOLDLINE)line)->text;
	}
	else {
		nchar -= LINEHEAD;
		instring = line->text;
	}
	if ( commandmode ) {
		position->num = column;
		if ( position->num > nchar ) {
			position->virt = position->num - nchar;
			position->num = nchar;
		}
		else position->virt = 0;
		position->address = line->text + position->num - 1;
		if ( position->virt > 0 ) {
			(position->virt)--;
			(position->address)++;
			if ( position->virt ) return(AFTERVIRTUAL);
			else return(NORMALEOL);
		}
		else return(NORMALCHAR);
	}
	tabstring = cbuf->tabstring;
	if ( cbuf->outputmode == RAWMODE ) tabstring = &zero;

	position->num = 0;
	while( (position->num)++ < nchar ) {
		if ( *instring == '\t' ) {
			while ( *tabstring && ( *tabstring < num ) ) tabstring++;
			position->virt = 0;
			if ( *tabstring ) {
				while ( num < *tabstring ) {
					if ( ++num == column ) {
						position->address = instring;
						return(INSIDETAB);
					}
					(position->virt)++;
				}
			}
			if ( ++num == column ) {
				position->address = instring;
				return(ONTAB);
			}
		}
		else {
			if ( ++num == column ) {
				position->virt = 0;
				position->address = instring;
				return(NORMALCHAR);
			}
		}
		instring++;
	}
	position->address = instring;
	position->num = nchar;
	if ( ++num == column ) {
		position->virt = 0;
		return(NORMALEOL);
	}
	position->virt = column - num;
	return(AFTERVIRTUAL);
}

/*
 		#] Column -> Character : 
 		#[ Character -> Column :	WORD FindColumn(line,number)
*/

WORD
FindColumn(LINE line,WORD number)
{
	WORD col = 0, nchar;
	UBYTE *tabstring;
	UBYTE *instring;

	if ( line->used < 0 ) line = (LINE)(line->text);
	nchar = line->used - LINEHEAD;
	instring = line->text;
	if ( commandmode ) tabstring = commandbuffer->tabstring;
	else {
		if ( cbuf->outputmode == RAWMODE ) {
			return(number);
		}
		tabstring = cbuf->tabstring;
	}

	if ( nchar < number ) { WORD a=number-nchar; number = nchar; nchar = a; }
	else nchar = 0;
	while ( --number >= 0 ) {
		if ( *instring++ == '\t' ) {
			while ( *tabstring && ( *tabstring < col ) ) tabstring++;
			if ( *tabstring ) { col = *tabstring; tabstring++; }
		}
		col++;
	}
	col += nchar;
	if ( col == 0 ) col = 1;
	return(col);
}

/*
 		#] Character -> Column : 
 		#[ Find line number :		LONG FindNum(line)
*/

LONG
FindNum(LINE line)
{
	LINE b, c;
	LONG num = 0;
	c = (LINE)cbuf;
	b = c->next;
	if ( ( b == c ) || ( line == c ) ) return(num);
	num++;
	if ( b == line ) return(num);
	do {
		if ( b->used < 0 ) num += LINESINFOLD(b);
		else			   num++;
		if ( ( b = b->next ) == line ) return(num);
	} while ( b != c );
	return(-1L);
}

/*
 		#] Find line number : 
 		#[ Count lines :			LONG CountLines(buffer)
*/

LONG
CountLines(BUFFER line)
{
	LINE buffer = (LINE)line;
	LINE c;
	LONG num = 0;
	c = buffer->next;
	while ( c != buffer ) {
		if ( c->used < 0 ) num += LINESINFOLD(c);
		else			   num++;
		c = c->next;
	}
	return(num);
}

/*
 		#] Count lines : 
 		#[ Count lines in range :	LONG NumLines(line1,line2)
*/

LONG
NumLines(LINE line1,LINE line2)
{
	LONG num = 0;
	while ( line1 != line2 ) {
		if ( line1->used < 0 ) num += LINESINFOLD(line1);
		else				   num++;
		line1 = line1->next;
	}
	return(num);
}

/*
 		#] Count lines in range : 
 		#[ Copy a line :			WORD CopyLine(from,after)
*/

WORD
CopyLine(LINE from,LINE after)
{
	WORD number;
	LONG *s, *t;
	if ( ( number = from->used ) < 0 ) {		/* Foldline */
		FOLDLINE line;
		number = ((FOLDLINE)from)->used - FOLDHEAD;
		line = (FOLDLINE)FoldAlloc(number,after);
		if ( line == 0 ) return(1);
		line->lines = ((FOLDLINE)from)->lines;
		line->mark = ((FOLDLINE)from)->mark;
		s = (LONG *)((FOLDLINE)from)->text;
		t = (LONG *)line->text;
		number = ( number + (sizeof(LONG)-1) ) / sizeof(LONG);
		while ( --number >= 0 ) *t++ = *s++;
		after = ((FOLDLINE)from)->first;
		from = ((FOLDLINE)from)->last->next;
		do {
			if ( CopyLine(after,line->last) ) {		/* No memory. Undo. */
				while ( ( after = line->first ) != (LINE)(&(line->first)) ) {
					LineDealloc(after);
				}
				LineDealloc((LINE)line);
				return(NOMEMORY);
			}
		} while ( ( after = after->next ) != from );
	}
	else {
		LINE line;
		number -= LINEHEAD;
		if ( ( line = LineAlloc(number,after) ) == (LINE)0 ) return(-1);
		s = (LONG *)from->text;
		t = (LONG *)line->text;
		number = ( number + (sizeof(LONG)-1) ) / sizeof(LONG);
		while ( --number >= 0 ) *t++ = *s++;
	}
	return(0);
}

/*
 		#] Copy a line : 
 		#[ Range of action :		WORD Range(st,ls,nst,nls,dvirt,dvcol)
*/

WORD
Range(LINE *start,LINE *last,WORD *numstart,WORD *numlast,WORD *diffvirt,WORD *diffvcol)
/*
 *	Determines the range of action in the current buffer.
 *	Return value is the order between cursor and mark or an error:
 *	1 : Mark -> cursor.
 *	0 : Cursor -> mark.
 *	-1: Error.
 *	numstart and numlast are the numbers of the characters in the
 *	corresponding lines.
 *	diffvirt is the difference in the number of virtual lines.
 *	diffvcol is the difference in the number of virtual positions
 *	inside a single line.
 */
{
	BUFFER b = cbuf;
	WORD mchar;
	LONG number;				/* 24-feb-1989 */
	FINDPOS fp;
	WORD nchar,nvirt = 0,type = 0;
	if ( commandmode ) b = commandbuffer;
	if ( b->mark == (LINE)b ) {
		MesnOut(35); /* "No mark." */
		return(-1);
	}
	if ( ( number = FindNum(b->mark) ) < 0 ) {
		MesnOut(7);	/* "Mark not found." */
		return(-1);
	}
	if ( b->MarkVirt > 0 ) {
		mchar = b->mark->used;
		if ( mchar < 0 ) mchar = ((FOLDLINE)(b->mark))->used - FOLDHEAD;
		else			 mchar -= LINEHEAD;
	}
	else {
		mchar = b->MarkChar;
	}
	if ( b->newvirt > 0 ) {
		nchar = b->current->used;
		if ( nchar < 0 ) nchar = ((FOLDLINE)(b->current))->used - FOLDHEAD;
		else			 nchar -= LINEHEAD;
	}
	else {
		type = FindChar(b->current,b->column,&fp);
		nchar = fp.num; nvirt = fp.virt;
	}
	if ( ( type == AFTERVIRTUAL ) || ( type == NORMALEOL ) ) nchar++;
	if ( ( b->MarkVoff > 0 ) && ( b->MarkVirt == 0 ) ) mchar++;
	if ( number > b->newline ) {
		*start = b->current;
		*last = b->mark;
		*diffvirt = b->MarkVirt - b->newvirt;
		*numstart = nchar;
		*numlast = mchar;
		*diffvcol = nvirt;
		return(0);
	}
	else if ( number < b->newline ) {
		*start = b->mark;
		*last = b->current;
		*diffvirt = b->newvirt - b->MarkVirt;
		*numstart = mchar;
		*numlast = nchar;
		*diffvcol = -nvirt;
		return(1);
	}
	*last = *start = b->current;
	if ( ( *diffvirt = b->newvirt - b->MarkVirt ) > 0 ) {
		*numstart = mchar;
		*numlast = nchar;
		*diffvcol = -nvirt;
		return(1);
	}
	else if ( *diffvirt < 0 ) {
		*diffvirt = -*diffvirt;
		*numstart = nchar;
		*numlast = mchar;
		*diffvcol = nvirt;
		return(0);
	}
	if ( mchar > nchar ) {
		*numstart = nchar;
		*numlast = mchar;
		*diffvcol = nvirt;
		return(0);
	}
	else if ( mchar < nchar ) {
		*numstart = mchar;
		*numlast = nchar;
		*diffvcol = -nvirt;
		return(1);
	}
	*numstart = *numlast = mchar;
	if ( b->MarkVoff > 0 ) {
		*diffvcol = nvirt - b->MarkVoff + 1;
		if ( ( *diffvcol >= 0 ) || ( type == INSIDETAB )
			|| ( type == ONTAB ) ) return(1);
		else { *diffvcol = -*diffvcol; return(0); }
	}
	if ( b->MarkVoff < 0 ) {	/* Mark inside tab */
		if ( ( type == NORMALEOL ) || ( type == AFTERVIRTUAL ) ) {
			*diffvcol = nvirt;
			return(1);
		}
		else if ( type == INSIDETAB ) {
			*diffvcol = b->column - b->MarkColumn;
			if ( *diffvcol >= 0 ) return(1);
			else { *diffvcol = -*diffvcol; return(0); }
		}
		else {
			*diffvcol = b->column - b->MarkColumn;
			return(1);
		}
	}
	if ( type == INSIDETAB )
		 { *diffvcol = b->MarkColumn - b->column; return(0); }
	*diffvcol = b->column - b->MarkColumn;
	return(1);
}

/*
 		#] Range of action : 
 		#[ Find a line :			LINE FindLine(number)
*/

LINE
FindLine(LONG number)
{
	BUFFER b = cbuf;
	LINE a1;
	LONG num = 0;
	if ( commandmode ) b = commandbuffer;
	a1 = b->first;
	if ( a1 == (LINE)b ) return(0L);
	do {
		if ( a1->used < 0 ) num += LINESINFOLD(a1);
		else			    num++;
		a1 = a1->next;
	} while ( num < number );
	a1 = a1->prev;
	if ( !commandmode ) {
		b->newline = number;
		b->newvirt = 0;
		b->current = a1;
	}
	return(a1);
}

/*
 		#] Find a line : 
  	#] Line utilities : 
  	#[ Dump Utility :
*/
#ifdef DEBUGON

VOID
DumpMem(VOID)
{
	MBLOCK m = FirstMem;
	LONG *lo;
	BYTE *s;
	WORD handle, j, k;
	if ( ( handle = CreateFile("dump.dmp",0) ) < 0 ) {
		MesOut("Cannot dump memory");
		return;
	}
	while ( m ) {
		lo = (LONG *)m;
		DoLong((LONG)lo,handle);
		FileWrite(handle,2L,"  ");
		for ( j = 0; j < 5; j++ ) {
			DoLong(*lo++,handle);
		}
		FileWrite(handle,2L,"\r\n");
		s = (BYTE *)lo;
		while ( ( k = ((LINE)s)->totlen ) > 0 && s < (BYTE *)(m->fill) ) {
			DoLong((LONG)lo,handle);		/* Write address */
			FileWrite(handle,2L,"  ");
			j = ((LINE)s)->used;
			DoLong(*lo++,handle);			/* 16 bytes */
			FileWrite(handle,1L," ");
			DoLong(*lo++,handle);
			FileWrite(handle,1L," ");
			DoLong(*lo++,handle);
			if ( j < 0 ) {
				FileWrite(handle,2L,"  ");
				DoLong(*lo++,handle);
				FileWrite(handle,1L," ");
				DoLong(*lo++,handle);			/* 16 more bytes */
				FileWrite(handle,1L," ");
				DoLong(*lo++,handle);
				s += 6*sizeof(LONG);
				k -= 6*sizeof(LONG);
			}
			else {
				s += 3*sizeof(LONG);
				k -= 3*sizeof(LONG);
			}
			FileWrite(handle,2L,"  ");
			FileWrite(handle,(LONG)k,s);
			FileWrite(handle,2L,"\r\n");
			s += k;
			lo += k / ((WORD)sizeof(LONG));
		}
		m = m->next;
	}
	CloseFile(handle);
}

VOID
DoLong(LONG x,WORD handle)
{
	BYTE buf[2*sizeof(LONG)];
	WORD i;
	for ( i = sizeof(LONG)-1; i >= 0; i-- ) {
		buf[i] = hex[x&0xF];
		x >>= 4;
	}
	FileWrite(handle,2*sizeof(LONG),buf);
}
#endif

/*
  	#] Dump Utility : 
  	#[ TestMem :
*/
#ifdef DEBUGON

LONG StillToGo;

LONG
TestMem(VOID)
{
	BUFFER b = cbuf;
	StillToGo = b->totline;
	if ( DoAfold(b->first,(LINE)b) ) {
		MesOut("Memory is corrupted");
	}
	else if ( StillToGo ) goto ToGo;
	else {
		StillToGo = b->totline;
		if ( DoBfold(b->last,(LINE)b) ) {
			MesOut("Memory is corrupted");
		}
		else if ( StillToGo ) {
ToGo:
			MesOut("Memory is too short");
		}
		else {
			MesOut("Memory is OK");
		}
	}
	return(0);
}

WORD
DoAfold(LINE l,LINE f)
{
	while ( l != f && StillToGo >= 0 ) {
		if ( l->used >= 0 ) {
			StillToGo--;
		}
		else {
			if ( DoAfold(((FOLDLINE)l)->first,(LINE)(((BYTE *)l)+LINEHEAD))
				< 0 ) return(-1);
		}
		l = l->next;
	}
	if ( StillToGo < 0 ) return(-1);
	return(0);
}

WORD
DoBfold(LINE l,LINE f)
{
	while ( l != f && StillToGo >= 0 ) {
		if ( l->used >= 0 ) {
			StillToGo--;
		}
		else {
			if ( DoBfold(((FOLDLINE)l)->last,(LINE)(((BYTE *)l)+LINEHEAD))
				< 0 ) return(-1);
		}
		l = l->prev;
	}
	if ( StillToGo < 0 ) return(-1);
	return(0);
}
#endif

/*
  	#] TestMem : 
*/

