/*
  	#[ Standard redefinitions :
#define CDEBUG
*/

#ifdef OPTERON
#define LINUX
#endif

#ifdef BSD
#define UNIX
#endif
#ifdef LINUX
#define UNIX
#define ANSI

#endif

#define WITHOUTLINE

#define MAXFILENAME 30
#define MAXFULLNAME 162

#include <stdio.h>
#include <stdlib.h>

#ifdef OPTERON

#define NUMBITS 64
#define WORD	int
#define LONG	long
#define BYTE	char
#define VOID	void
#define BOOLEAN int
#define FILLSIZE 8
#define MEMLENGTH 1048576
#define READLENGTH 524288

#else

#define NUMBITS 32
#define WORD	short
#define LONG	long
#define BYTE	char
#define VOID	void
#define BOOLEAN int
#define FILLSIZE 4
#define MEMLENGTH 16384
#define READLENGTH 8192

#endif

#define UWORD	unsigned WORD
#define ULONG	unsigned LONG
#define UBYTE	unsigned BYTE
 
typedef UBYTE	*STRENG;

#ifdef ANSI
typedef VOID	*PVOID;
#else
typedef LONG	PVOID;
#endif

#ifdef ANSI
typedef WORD	(*WFUN)(BYTE *);
typedef LONG	(*PfunLo)(VOID);
typedef LONG	(*PfunLoP)(UBYTE *,WORD);
typedef VOID	(*VFUN)(UBYTE *,WORD);
typedef VOID	(*VFUNW)(WORD);
typedef VOID	(*VFUNWW)(WORD,WORD);
#ifndef NULL
#define NULL (VOID *)0
#endif
#else
typedef WORD	(*WFUN)();
typedef VOID	(*VFUN)();
typedef VOID	(*VFUNW)();
typedef VOID	(*VFUNWW)();
typedef LONG	(*PfunLo)();
typedef LONG	(*PfunLoP)();
#ifndef NULL
#define NULL 0
#endif
#endif

#define P_LO(x) ((ULONG)(x))
#define TOLONG(x) ((LONG)(x))

/*
  	#] Standard redefinitions :
  	#[ From osbind :

	All system dependencies are in the following routines and the 
	fold labelled 'Operating system dependent routines'.
	All routines are commented here to facilitate portability.
	They are used here for the Atari ST series.

	(WORD)GetDrive()
		returns the number of the current drive. 0 = 'A' etc.

	(VOID)SetNewDTA(PointerToNewDTABuffer)
		DTA PointerToNewDTABuffer
		Makes that in the future the given buffer is used.
		This is of importance for searchactions with FindFile.
		The layout of the DTA buffer is given in the definition
		of the struct dtabuf.

	(LONG)Super(Oldstack)
		LONG Oldstack;
		Used to switch to and from supervisor mode. When Oldstack is
		zero it causes entry into the supervisor mode and returns
		the value of the old supervisor stack. When called again
		to go back this value should be provided or there will be
		a crash.

	(LONG)CreateFile(name,attribute)
		BYTE *name; WORD attribute;
		attribute = 0 -> read/write. Other values are not of interest here.
		Returns either a handle (WORD) or a LONG negative number.
		Negative WORD handles are for the standard devices.

	(LONG)OpenFile(name,attribute)
		BYTE *name; WORD attribute;
		attribute = 0 -> read only. Other values are not of interest here.
		Returns either a handle (WORD) or a LONG negative number.
		Negative WORD handles are for the standard devices.

	(LONG)CloseFile(handle)
		WORD handle;
		Closes a file that was previously opened or created.
		The return value is zero if everything went OK.

	(LONG)FileRead(handle,number,buffer)
		WORD handle; LONG number; BYTE *buffer;
		Tries to read 'number' bytes into the given buffer.
		The return value is either the number of actually read bytes
		or a negtive number indicating an error.

	(LONG)FileWrite(handle,number,buffer)
		WORD handle; LONG number; BYTE *buffer;
		Tries to write 'number' bytes from the given buffer.
		The return value is either the number of actually written bytes
		or a negtive number indicating an error.

	(LONG)FileRemove(name)
		BYTE *name;
		Either the given file is deleted/removed/unlinked or a negative
		errornumber is returned.

	(LONG)FileType(name,mode,attribute)
		BYTE *name; WORD mode,attribute;
		mode = 0	Ask for attribute/filetype of file 'name'.
		mode = 1	Set filetype to 'attribute'.
		The value of attribute is according to its bits set:
		0	Set to read only.
		1	Hidden file
		2	System file
		3	Volume label
		4	Subdirectory
		5	Archived file.
		The return value is either the attribute or a negative errormessage.

	(VOID)GetPath(buffer,drive)
		BYTE *buffer; WORD name;
		Buffer is at least 64 bytes. This function gives the pathname
		of the current directory on 'drive' which has the value 0 for the
		current drive, 1 for 'A' etc. The root directory corresponds to
		a NULL string.

	(BYTE *)Malloc(number)
		LONG number;
		Returns either a pointer to a block of memory of size 'number'
		that the user gets allocated or a value <= 0. When number is
		-1 it returns a LONG value indicating the total amount of free
		memory.

	(LONG)Mfree(address)
		(BYTE *)address;
		Returns the memory starting at address to the systems pool. This
		address must have been obtained previously with a call to Malloc.

	(WORD)FindFile(name[,dta],attribute)
		BYTE *name; WORD attribute;
		Looks for all files of which the name matches the one given,
		including the wildcards. The value of attribute is used for a
		preselection so that only files of the proper type will give
		a match. If a file is found the return value is zero and its
		parameters are given in the current dta buffer.

	(WORD)FileRename(dummy,oldname,newname)
		WORD dummy; BYTE *oldname, *newname;
		Changes the name of the file with 'oldname' to 'newname'.
		If this wasn't possible the returnvalue is negative. Zero indicates
		that everything went as should.

*/

typedef struct dtabuf {
	UBYTE reserved[21];
	UBYTE attribute;
	UWORD time;
	UWORD date;
	LONG  size;
	UBYTE name[MAXFILENAME+2];
} *DTA;

#ifdef UNIX

 #include <fcntl.h>
 #include <unistd.h>

 extern long lseek();

 #define CreateFile(name,y)	open(name,(int)(O_RDWR | O_CREAT | O_TRUNC),(int)0666)
 #define OpenFile(name,y)	open(name,(int)O_RDONLY,(int)0666)
 #define CloseFile(handle)	close((int)(handle))
 #define FileRead(handle,count,buffer)	read((int)(handle),(char *)buffer,(int)(count))
 #define FileWrite(handle,count,buffer)	write((int)(handle),(char *)buffer,(int)(count))
 #define FileRemove(name)	unlink((name))

 #define FileSeek(offset,handle,mode)	lseek((int)(handle),(long)(offset),(int)(mode))

 #define Malloc(x)			malloc((unsigned int)(x))
 #define MALLOC(x)			malloc((unsigned int)(x))
 #define Mfree(x)			{ if ( x ) free(x); }

 #define FileRename(x,y,z)	rename(y,z)
 #define Cconws(x)			write(1,x,strlen(x))
 #define Cconout(x)	{ char y = x; write(1,1L,&y); }
 #define GetPath(x,y) getcwd(x,y)
 #define Bconin(x) getchar()

 #define WINDOWS
 #define HWINDOWS

 #define SPACEBIT '.'
 #define TABBIT '>'
 #define ZEROBIT '_'
 #define GRAVECROSS 'x'
 #define PROMPTC ':'
 #define LESSLESS '{'
 #define LARGLARG '}'
 #define INFTY 'o'
 #define DIRTYBIT '.'
 #define WAITBIT '?'

 #define SEPARATOR '/'
 #define WITHBREAK
 #define BREAKCODE (UBYTE)3

 #define NEWLINE 0x0A
 #define CARRETURN 0x0D

 #define LINENUMBERDEFAULT 0
 #define DEFOUTPUTMODE UNIXMODE

 #define OWNENV
 #define MYGETENV(x) mygetenv(x)

 #define WITHBACKSLASH

 #define WITHDATE
 #define WITHSYNCH

#endif

/*
  	#] From osbind :
  	#[ Parameters :
*/

#define REGMEMSIZE 1000L

#define MUSTDRAW 1
#define NODRAW -5
#define UPDATE 0

#define INSERTMODE 0
#define OVERSTRIKEMODE 1
#define RAWMODE 0
#define UNIXMODE 1
#define ATARIMODE 2
#define PRINTMODE 3
#define FORMATMODE 4
#define NOTSENSITIVE 0
#define SENSITIVE 1
#define VIEWONLY 0
#define VISIBLEBACKUP 1
#define HIDDENBACKUP 2
#define NOBACKUP 3
#define CIRCULAR 0

#define NORMALEOL 0
#define NORMALCHAR 1
#define INSIDETAB 2
#define AFTERVIRTUAL 3
#define ONTAB 4

#define INTEXT 0
#define INCOMMAND 1

#define FILENOTFOUND -33
#define NOMEMORY -39
#define NOMEMORYL -39L

#define COLUMNSTEP 20
#define MAXCOLUMN 255
#define MAXVIRT 5
#define MAXLEARN 100
#define MAXLEFT 10
#define MAXINLINE 255
#define MAXCOMMAND 256
#define MAXIPATH 64
#define MAXTAIL 126
#define MAXMENU 100

#define NUMTAGS 10

#define READONLY 0

#define TIMEOUT 20000

#define UNDOCODE 10
#define UNDOBUFFER 4
#define UNDOLINE -1
#define UNDOPART 1
#define UNDOEOL 2
#define NONAMES 125

#define CONTIN 2
#define YES 1
#define NO 0
#define QUIT -1

#define VETO 1
#define FORWARDMODE 1
#define BACKWARD -1

#define ON 1
#define OFF 0

#define A_NORMAL 0
#define A_INVERSE 1
#define A_CLEAR 2
#define A_ALL 3

#define HORIZONTAL 1
#define VERTCAL 2

#ifdef WITHMASKS
#define MASKS 1
#else
#define MASKS 0
#endif

#define MAXBRACKET 10
#define MAXVARS 300
#define MREADLENGTH 1024
#define MMEMLENGTH 1300
#define SCRAPSIZE 256

#define MAXNUMWINDOWS 2

/*
  	#] Parameters :
  	#[ Info structs :

	Note: the size of struct tline should be a power of 2.
	The same holds for struct fline. Usually twice sizeof(struct tline)
*/

typedef struct tline {
	struct tline *next;	 /* Pointer to next line */
	struct tline *prev;	 /* Pointer to previous line */
	WORD totlen;		 /* Number of total allocation */
	WORD used;			 /* Number of bytes used in total */
	UBYTE text[FILLSIZE];/* Note that this may be longer of course */
				} *LINE;

typedef struct fline {
	struct tline *next;		/* Pointer to next line */
	struct tline *prev;		/* Pointer to previous line */
	WORD totlen;			/* Number of total allocation */
	WORD mark;				/* -1 to indicate foldline */
	struct tline *first;	/* Pointer to previous line */
	struct tline *last;		/* Pointer to next line */
	UWORD lines;			/* Number of lines inside */
	WORD used;				/* Number of bytes used in total */
	UBYTE text[2*FILLSIZE];	/* Note that this may be longer of course */
				} *FOLDLINE;

#define LINESIZE (NUMBITS/2)
#define FOLDLINESIZE (NUMBITS/2)

#define LINEHEAD ((WORD)sizeof(struct tline)-FILLSIZE)
#define FOLDHEAD ((WORD)sizeof(struct fline)-2*FILLSIZE)

struct tbuff {
	struct tline *first;		/* First line in buffer */
	struct tline *last;			/* Last line in buffer */
	struct tline *current;		/* The current line */
	struct tline *mark;			/* The line with the mark */
	WORD undo;					/* For the undo buffer only. Type of undo. */
	WORD bufnum;				/* Number of buffer - 1. Position in struct fixed! */
	WORD MarkColumn;			/* Column with the mark */
	WORD MarkVirt;				/* Number of virtual positions for the mark */
	WORD MarkChar;				/* Number of the character with the mark */
	WORD MarkVoff;				/* Virtual offset (tabs etc) */
	WORD screenline;			/* Number of line on screen with cursor */
	WORD column;				/* Column with the cursor */
	WORD offset;				/* Horizontal displacement of screen */
	WORD screencolumn;			/* Number of column on screen with cursor */
	LONG oldline;				/* Previous line number (screen not updated) */
	LONG newline;				/* Line number (screen not updated) */
	LONG totline;				/* Number of lines in buffer */
	WORD oldvirt;				/* Previous number of virtual lines */
	WORD newvirt;				/* Number of virtual lines */
	WORD iomode;				/* Insert(0)/Overstrike(1) mode */
	WORD outputmode;			/* R(0),U(1),A(2),P(3),F(4) */
	WORD direction;				/* <(-2),<<(-1),0(0),>>(1),>(2) */
	WORD sensitivity;			/* N(0),S(1) */
	WORD backupmode;			/* V(0),B(1),b(2),NoBackup(3) */
	WORD dirtyflag;				/* Was file modified (Y=1, N=0) */
	WORD wordwrap;
	WORD autoindent;
	UBYTE tabstring[100];
	UBYTE name[MAXFILENAME+2];
	UBYTE fullname[MAXFULLNAME+2];
				};
typedef struct tbuff *BUFFER;

#define MHEADLENGTH 2*(sizeof(WORD)+ sizeof(LINE)+sizeof(struct tmemory *))
#define DATALENGTH MEMLENGTH-MHEADLENGTH

typedef struct tmemory {
	struct tmemory *prev;
	struct tmemory *next;
	LINE user;
	LINE fill;
	WORD SpaceAtEnd;
	WORD AllSpace;
	UBYTE cdata[16];
				} *MBLOCK;

#define NO_SPECIAL_VERSION
/* DEFVERSION is the version of the default file. If anything is changed
   in the layout of the default file, this number should be raised by one.
   There have been some spectacular errors when structs were read in
   wrong from this file. */
#define DEFVERSION 71
#define VERSION_MESSAGE "STedi version 3.1"
#define MESSAGE1	"Cursor not inside a legal fold."
#define MESNUM1		31
#define MESSIZE1    32
#define MESSAGE2	"Cursor not in a fold line."
#define MESNUM2		32
#define MESSIZE2	27
#define MESSAGE3	"No reading in fold line"
#define MESNUM3		15
#define MESSIZE3    24
#define MESSAGE4	"No memory for writing."
#define MESNUM4		20
#define MESSIZE4	23

typedef	struct defbuf {
	UWORD version;
	WORD color;
	WORD iomode;
	WORD outputmode;
	WORD direction;
	WORD sensitivity;
	WORD backupmode;
	WORD wordwrap;
	WORD autoindent;
	WORD maxhist;
	WORD maxinpage;
	WORD printoffset;
	UWORD PageWait;
	UWORD color1;					/* Window color */
	UWORD color2;					/* Command line color */
	WORD lnumber;
	WORD waitflag;
	WORD autoclose;
	WORD DoBIOS;
	UWORD macros;					/* Are there macro's ? */
	WORD menuversion;
	WORD menunotspaced;
	LONG redeflength;				/* Length of redefinition file */
	LONG numredef; 					/* Number of keyredefinitions */
	UBYTE menu[MAXMENU];
	BYTE printer[8];
	BYTE wordtable[256];
	UBYTE tabstring[100];
	WORD InLearnBuffer[10];
				} *DEFAULT;

typedef struct Etags {
	LINE line;
	WORD bufnum;
	WORD column;
} TAG;

typedef struct s_rect {
	WORD r1, c1;		/* upper left corner */
	WORD r2, c2;		/* lower right corner */
	WORD hi, wi;		/* height and width */
	WORD ro, co;		/* row and column offsets */
	WORD bufnum;
	WORD att;
#ifdef WITHMASKS
	LONG Lmask;			/* mask on left side */
	LONG Rmask;			/* mask on right side */
	WORD offset;		/* offset to first byte left */
	WORD NumLong;		/* number of longs to be copied without mask */
	WORD inc;			/* number of bytes not touched */
	WORD dummy;			/* For allignment. */
#endif
} SCREEN;

typedef struct macro_s {
	struct macro_s *next;
	LONG           size;	/* size of the whole contents */
	LONG           start;	/* position from next where the text starts */
	WORD           used;
	UBYTE          ownmalloc;
	UBYTE		   namesize;
	UBYTE          name[4];
} *MACRO;
 
typedef struct vari_ {			/* For evaluation purposes */
	UBYTE *where;
	UBYTE *inbuf;
	LONG number;
	WORD operation;
	WORD bracket;
} VARIABLE;

typedef struct specs {
	BYTE *key;
	PfunLo fun;
} SPECS;

typedef struct {				/* For calling FindChar */
	UBYTE *address;
	WORD num;
	WORD virt;
} FINDPOS;

/*
  	#] Info structs :
  	#[ Macros :
*/

#define LENGTH(l) ((l)->used<0)?(((FOLDLINE)(l))->used-FOLDHEAD):((l)->used-LINEHEAD)
#define SHOWRESULT if(commandmode){MessageCommand();}else{UpdateDisplay();}
#define SKIPBLANKS(a,b) while((b>0)&&(*a==' ')){b--;a++;}
#define SKIPTBLANKS(a,b) while((b>0)&&( *a == ' ' || *a == '\t')){b--;a++;}
#define SKIPEBLANKS(a,b,c) while((b>0)&&( *a == ' ' || *a == c )){b--;a++;}
#define TABLESIZE(a,b) (((WORD)sizeof(a))/((WORD)sizeof(b)))

#define NCOPY(s,t,n) while (--n>=0) { *s++ = *t++; }

/*	DEFSCREEN(&Screen,row1,row2,col1,col2,att) */
#define DEFSCREEN(a,b,c,d,e,f) ((a)->r1=b,(a)->r2=c,(a)->c1=d,(a)->c2=e,(a)->att=f)
/*	DEF2SCREEN(&Screen) */
#define DEF2SCREEN(a) ((a)->hi=(a)->r2-(a)->r1+1,(a)->wi=(a)->c2-(a)->c1+1,\
(a)->ro=(a)->r1-1,(a)->co=(a)->c1-1)

#if ( NUMBITS == 32 )

#define LINESINFOLD(l) ((((LONG)(((LINE)(l))->used&0x3FFF))<<16)+((FOLDLINE)(l))->lines)
/* #define LINESINFOLD(l) (((FOLDLINE)(l))->lines) */
#define FOLDCODE ((WORD)0xC000)

#define EXITCODE 0x97654321L

#endif
#if ( NUMBITS == 64 )

#define LINESINFOLD(l) ((((LONG)(((LINE)(l))->used&0x3FFFFFFF))<<32)+((FOLDLINE)(l))->lines)
/* #define LINESINFOLD(l) (((FOLDLINE)(l))->lines) */
#define FOLDCODE ((WORD)0xC0000000)

#define EXITCODE 0x9765432197654321L

#endif

/*
  	#] Macros :
  	#[ Variables :
*/

extern SCREEN Screen;
extern SCREEN *TextScreen;
extern SCREEN Mes1Screen;
extern SCREEN Mes2Screen;
extern SCREEN StatScreen;
extern SCREEN SplitScreen;
extern SCREEN TScreen[];
extern WORD ActiveScreen;
extern WORD SplitType;

extern BUFFER textbuffer[];		/* 0-7 Text, 8-9 Yank, 10 Undo, 11 Commands */
extern BUFFER commandbuffer;
extern BUFFER cbuf;
extern BUFFER undoB;
extern WORD redraw;
extern WORD undone;
extern TAG tags[];

extern UBYTE *bstopper;
extern UBYTE *bstop2;
extern UBYTE *bufRead;

extern WORD ScreenWidth;
extern WORD ScreenLength;
extern WORD ScrollStep;
extern LONG BytePerLine;		/* Bytes in one textline */
extern WORD FullLines;			/* Number of lines if no command line */

extern WORD learning;
extern WORD LearnReplay;
extern WORD NumLearnBuffer[];
extern WORD MacroReplay;
extern LONG *FromMacro;
extern LONG MacroFlags;
extern LONG *KeyPointer;
extern LONG NumberKeys;
extern LONG KeyStack[];

extern WORD commandmode;
extern WORD hexmode;
extern WORD HexBuffer;
extern UBYTE CommandText[];
extern UBYTE Com2Text[];
extern WORD WhichYank;
extern UBYTE MessageText[];		/* MAXSCREENWIDTH */
extern UBYTE LastMessage[];
extern WORD InLastMess;
extern UBYTE *MessagePointer;
extern WORD MinCommandColumn;
extern WORD BlockExit;
extern UBYTE *WordContinue;
extern WORD WordCn;

extern UBYTE ToUpper[];
extern UBYTE ToLower[];
extern BYTE wordtable[];
extern UBYTE hex[];
extern UBYTE CtrAbuf[];
extern WORD CtrAlen;
extern WORD HexOutMode;
extern WORD HexSetting;

/*
extern UBYTE RegMem[];
*/

extern UBYTE defaultdir[];
extern struct defbuf dbufFer;
extern WORD LearnBuffer[];

extern WORD TabShow;
extern WORD BlockScreen;
extern WORD DoNumber;

extern DTA dta;
extern WORD InMustWrite;
extern UBYTE MustWrite[];
extern UBYTE *MustWpoin;
extern WORD InMustW;
extern UBYTE OldRead[];
extern WORD OldReadN;

extern WORD OldKeyValue;
extern WORD NewKeyValue;

extern WORD oldbuffer;

extern WORD Cursor;
extern WORD ColumnStep;
extern WORD cColumnStep;

extern UBYTE *HelpName;
extern BYTE HlpName[];
extern BYTE DefName[];
extern UBYTE plural[];

#ifdef WITHMOUSE
extern WORD VisMouse;
extern WORD OnMouse;
extern WORD MenuVisible;
extern BYTE PathFS[];
extern WORD inmenu;
#endif

extern WORD AskDump;
extern UBYTE *DirKBshift;

extern LONG LineToGo;
extern WORD AppendFlag;
extern LONG StartPos;
extern WORD Scol3;
extern WORD Scol4;
extern WORD ColMode;
extern WORD DispBlock;

extern LONG InInputBuf;

extern DTA sdta;
extern WORD dtaused;
extern WORD dtacont;

extern BUFFER varbuf;
extern MACRO FirstMacro;
extern UBYTE varbuffer[], *buftop, *inbuf;
extern UBYTE scrap[];
extern WORD inscrap;
extern WORD gargc;
extern WORD startupmacro;
extern UBYTE **gargv;
extern BYTE *MacroText;
extern WORD MacroTnum;

#ifdef UNIX
extern int rwflag;
extern int winupflag;
#endif

/*
  	#] Variables :
  	#[ Functions :
*/

#ifdef ANSI

 LONG MyOpen(UBYTE *, WORD);
 LONG MyCreate(UBYTE *, WORD);
 WORD MyRename(WORD, UBYTE *, UBYTE *);
 WORD Sfirst(UBYTE *,DTA,WORD);
 VOID IniKbuffer(VOID);
 VOID InStall(VOID);
 VOID ReStall(VOID);
 WORD ChMod(UBYTE *, WORD, WORD);
 WORD MyRemove(UBYTE *);
 WORD backup(UBYTE *,WORD);
 LINE InsertBuffer(LINE,UBYTE **);
 LONG ReadFile(UBYTE *);
 LONG ReadC(UBYTE *, WORD);
 LONG readkey(VOID);
 LONG WriteLine(LINE);
 LONG WriteFile(UBYTE *, WORD);
 LONG WriteC(UBYTE *, WORD, WORD);
 LONG writekey(VOID);

 LONG FilePrint(WORD, LONG, UBYTE *);
 LONG Print(UBYTE *, WORD);
 LONG klaar(VOID);
 LONG save(VOID);
 LONG savek(VOID);
 LONG SaveS(UBYTE *, WORD);
 LONG Quit(UBYTE *, WORD);
 LONG Yank(UBYTE *, WORD);
 LONG Get(UBYTE *, WORD);
 LONG ErrorMes(LONG, WORD);
 LONG FilToDef(WORD);
 LONG DefToFil(VOID);
 VOID BufToDef(VOID);
 VOID DefToBuf(VOID);
 VOID ShowDef(VOID);
 VOID SetDef(UBYTE *, WORD);
 VOID setdefdir(UBYTE *);
 LONG Dfaults(UBYTE *, WORD);
 LONG help(VOID);

 WORD ExpandLine(WORD,WORD,UBYTE *,UBYTE *,UBYTE *,WORD);
 VOID PutOut(WORD,WORD,LINE);
 VOID LineDisplay(LINE);
 VOID Display(WORD);
 VOID UpdateDisplay(VOID);
 VOID ForceDisplay(VOID);
 VOID DoDisplay(VOID);
 WORD Distance(LONG);
 VOID CursorOn(VOID);
 VOID CursorOff(VOID);
 VOID TextCursor(VOID);
 LONG Syntax(VOID);
 LONG NoFold(VOID);
 LONG NoMark(VOID);
 VOID MessageRedo(UBYTE *);
 WORD MessageWindow(SCREEN *,UBYTE *,UBYTE *);
 VOID MessageUpdate(VOID);
 VOID MessageCommand(VOID);
 VOID MessageForce(VOID);
 VOID MesOut(BYTE *);
 VOID MesMem(VOID);
 
 VOID MesnOut(WORD);
 VOID MesIll(WORD);
 VOID MesPrint(WORD n, ... );
 WORD MesAsk(WORD,WORD);
 VOID MesCon(BYTE *);
 VOID MesNumS(LONG);
 VOID MesNum(LONG);
 LONG RecallMessage(VOID);
 VOID ScrollUp(WORD,LINE);
 VOID ScrollDown(WORD,LINE);
 VOID PullUp(WORD,LINE);

 VOID Initialize(VOID);
 VOID inidef(VOID);
 VOID SetScreen(WORD);
 VOID ResetScreen(VOID);
 VOID givename(BUFFER,UBYTE *,UBYTE *);
 VOID inibuf(WORD);
 VOID namfil(UBYTE *);
 WORD GetOption(UBYTE **);
 VOID exit(int);
 VOID SetHelp(UBYTE *);

 VOID Strcpy(UBYTE *, UBYTE *);
 VOID strNcpy(UBYTE *, UBYTE *, WORD);
 WORD strNcmp(UBYTE *, UBYTE *, WORD);
 WORD strNIcmp(UBYTE *, UBYTE *, WORD);
 WORD StrCmp(UBYTE *, UBYTE *);
 WORD strcmp1(UBYTE *, UBYTE *);
 WORD StrCIcmp(UBYTE *, UBYTE *);
 WORD ReadHex(WORD, WORD);
 #ifdef OWNENV
  BYTE *mygetenv(BYTE *);
 #endif
 VOID LtoA(LONG,UBYTE *);
 LONG ItoL(UBYTE *,WORD);
 UBYTE *ReadNum(UBYTE *,WORD *,WORD *);
 WORD DetWor(UBYTE **);
 WORD GetRange(LINE *,LINE *,WORD);
 LONG Trim(WORD);
 LONG Tabbing(WORD);
 LONG DeTab(WORD);
 LONG Tab(UBYTE *, WORD);
 VOID StrConv(UBYTE *, WORD,UBYTE *);
 LONG Convert(UBYTE *);
 LONG upper(VOID);
 LONG lower(VOID);
 LONG flip(VOID);
 LONG FlipLines(VOID);
 LONG SetSome(UBYTE *, WORD);
 WORD DoDate(UWORD,UWORD);
 WORD MakeDate(UBYTE *);

 WORD StartMem(VOID);
 LINE LineAlloc(WORD,LINE);
 FOLDLINE FoldAlloc(WORD,LINE);
 VOID LineDealloc(LINE);
 VOID DropPart(LINE,WORD);
 LONG garb(VOID);
 WORD Garbage(VOID);
 VOID CleanBlock(MBLOCK);
 WORD FillBlock(MBLOCK);
 VOID MoveLine(LINE, LINE);
 VOID MakeRMO(MBLOCK);
 WORD ColToChar(LINE,WORD);
 WORD FindChar(LINE,WORD,FINDPOS *);
 WORD FindColumn(LINE, WORD);
 LONG FindNum(LINE);
 LONG CountLines(BUFFER);
 LONG NumLines(LINE, LINE);
 WORD CopyLine(LINE, LINE);
 WORD Range(LINE *,LINE *,WORD *,WORD *,WORD *,WORD *);
 LINE FindLine(LONG);

 VOID iniundo(VOID);
 VOID Discard(LINE);
 VOID ClearUndo(VOID);
 VOID CopSet(WORD, WORD, WORD);
 VOID ExchSet(WORD);
 VOID BufUnd(WORD);
 LONG empty(VOID);
 LONG AddUndo(LINE,WORD);
 LONG undo(VOID);

 VOID Bstrout(BYTE *);
 VOID MakeMasks(SCREEN *);
 VOID DefineWindow(SCREEN *,WORD,WORD,WORD,WORD,WORD,WORD);
 VOID PSbegin(VOID);
 VOID PSend(VOID);
 VOID PSwrite(SCREEN *,WORD,WORD,WORD,BYTE *,WORD);
 VOID PSsetcur(SCREEN *,WORD, WORD);
 VOID PSshowcur(WORD);
 VOID PSbeep(VOID);
 VOID PSscroll(SCREEN *,WORD,WORD,WORD);
 VOID PSclreol(SCREEN *,WORD,WORD);
 VOID PSclear(SCREEN *);
 VOID PSrectclear(WORD,WORD,WORD,WORD,WORD);
 VOID PScolor(VOID);
 VOID PSsynch(VOID);
 UBYTE *TakeScreen(LONG);
 WORD Excute(BYTE *,BYTE *,BYTE *,LONG *);
 WORD PrintOne(WORD,BYTE);
 WORD KeyPoll(WORD);
 LONG KeyGet(VOID);
 LONG KeyChar(VOID);
 LONG KeyRead(VOID);
 VOID KeyFlush(VOID);
 WORD SetKey(WORD);
 VOID KeyIni(VOID);
 WORD KeyStart(VOID);
 VOID KeyEnd(VOID);
 LONG GetTicks(VOID);
 LONG Exec(WORD, UBYTE *, UBYTE *, UBYTE *);
 WORD IsAvailable(WORD);
 LONG DiskFree(UBYTE *, WORD);
 WORD IsBrake(VOID);
 WORD IsCaps(VOID);
 VOID MyWait(LONG);
 #ifdef WITHMOUSE
  VOID CursorBlink(WORD);
 #endif

 LONG Cinsert(UBYTE *,WORD);
 LONG Linsert(VOID);
 LONG InNorm(UBYTE *,WORD);
 LONG CRnorm(VOID);
 LONG Tdelete(VOID);
 LONG Cdelete(VOID);
 LONG Bdelete(VOID);
 LONG Ldelete(VOID);
 LONG setmrk(VOID);
 LONG remomk(VOID);
 LONG gotomk(VOID);
 LONG exgmrk(VOID);
 LONG twidle(VOID);
 LONG deleol(VOID);
 LONG insert(VOID);
 LONG append(VOID);
 LONG puth(VOID);
 VOID DetMark(VOID);
 VOID DetMarkColumn(VOID);
 VOID ClearHexMode(VOID);

 VOID LearnEnter(LONG);
 WORD GetLearn(VOID);
 LONG StartLearn(UBYTE *,WORD);
 VOID QuitReplay(VOID);
 VOID QuitReplay(VOID);
 VOID UpdateAll(VOID);
 VOID RedefScroll(VOID);
 LONG GetMacro(VOID);
 LONG GetAkey(WORD);
 LONG Input(UBYTE *,WORD);
 LONG FlipSet(UBYTE *,WORD);
 LONG FlipScreen(VOID);
 WORD maskcmp(LONG,LONG);

 LONG l_arrow(VOID);
 LONG sl_arr(VOID);
 LONG cl_arr(VOID);
 LONG shctle(VOID);
 LONG r_arrow(VOID);
 LONG sr_arr(VOID);
 LONG cr_arr(VOID);
 LONG shctri(VOID);
 LONG u_arrow(VOID);
 LONG su_arr(VOID);
 LONG ctrup(VOID);
 LONG d_arrow(VOID);
 LONG sd_arr(VOID);
 LONG ctrdo(VOID);
 LONG clrhom(VOID);
 LONG sh_hom(VOID);
 LONG cthome(VOID);
 LONG ctlhom(VOID);
 LONG gotoln(LONG);
 LONG chline(LONG);
 LONG MoveColumn(UBYTE *,WORD);
 LONG MoveLast(UBYTE *,WORD);
 LONG MovePlus(UBYTE *,WORD);
 LONG MoveMin(UBYTE *,WORD);
 LONG MovePoint(UBYTE *,WORD);
 LONG MoveCursor(UBYTE *,WORD);
 LONG GotoTag(UBYTE *,WORD);
 LONG PutTag(UBYTE *,WORD);
 VOID ToEOL(VOID);
 VOID UpOne(VOID);
 VOID DownOne(VOID);
 WORD Distend(WORD);

 LINE OpenFold(FOLDLINE);
 FOLDLINE CloseFold(LINE,LINE);
 LINE FindStart(LINE,WORD *);
 LINE FindEnd(LINE,WORD *);
 WORD FindFold(LINE *,LINE *);
 LONG CloseCFold(VOID);
 LONG OpenCFold(VOID);
 LONG OpenAll(VOID);
 LONG CloseAll(VOID);
 LONG AutoClose(VOID);
 LONG OpenNum(LONG);
 LONG CloseNumber(LONG);
 LONG OpenName(UBYTE *,WORD);
 LONG CloseName(UBYTE *,WORD);
 LONG COpenFold(UBYTE *,WORD);
 LONG CCloseFold(UBYTE *,WORD);
 VOID RemoveFold(LINE);

 LONG CountBytes(LINE,LINE);
 LONG Where(VOID);
 LONG rawkey(VOID);
 LONG intkey(VOID);
 LONG OneKeyUp(PfunLo);
 LONG FindBuffer(UBYTE *,WORD);
 LONG pathnam(VOID);
 LONG set_sbr(VOID);
 LONG set_raw(VOID);
 LONG tog_hexset(VOID);
 LONG tog_tab(VOID);
 LONG set_ynk(VOID);
 LONG set_auu(VOID);
 LONG set_ioi(VOID);
 LONG set_ioo(VOID);
 LONG set_prn(VOID);
 LONG set_aua(VOID);
 LONG set_sns(VOID);
 LONG set_sdr(VOID);
 LONG togcol(VOID);
 LONG set_snn(VOID);
 LONG setview(VOID);
 LONG setback(VOID);
 LONG tog_io(VOID);
 LONG tog_ns(VOID);
 LONG tog_au(VOID);
 LONG tog_ua(VOID);
 LONG deadzone(VOID);
 LONG unsetv(VOID);
 LONG version(VOID);
 LONG HorStep(UBYTE *,WORD);
 LONG GotoBuffer(WORD);
 LONG alt1(VOID);
 LONG alt2(VOID);
 LONG alt3(VOID);
 LONG alt4(VOID);
 LONG alt5(VOID);
 LONG alt6(VOID);
 LONG alt7(VOID);
 LONG alt8(VOID);
 LONG alt9(VOID);
 LONG alt0(VOID);
 LONG control(WORD);
 LONG ctrl1(VOID);
 LONG ctrl2(VOID);
 LONG ctrl3(VOID);
 LONG ctrl4(VOID);
 LONG ctrl5(VOID);
 LONG ctrl6(VOID);
 LONG ctrl7(VOID);
 LONG ctrl8(VOID);
 LONG ctrl9(VOID);
 LONG ctrl0(VOID);
 LONG unknown(VOID);
 LONG unknown2(UBYTE *,WORD);
 LONG ScreenU(UBYTE *,WORD);
 LONG Tag1(VOID);
 LONG Tag2(VOID);
 LONG Tag3(VOID);
 LONG Tag4(VOID);
 LONG Tag5(VOID);
 LONG Tag6(VOID);
 LONG Tag7(VOID);
 LONG Tag8(VOID);
 LONG Tag9(VOID);
 LONG Tag0(VOID);
 LONG creturn(VOID);
 LONG lfeed(VOID);
 LONG Jump(LONG);
 LONG Alttw(UBYTE *,WORD);
 LONG CtrlC(UBYTE *,WORD);
 LONG FunKey(UBYTE *,WORD);
 SPECS *SpecKey(UBYTE *,WORD);
 LONG Command(UBYTE *,WORD);
 LONG repeat(VOID);
 LONG repeat2(VOID);
 LONG ForCommand(WORD);
 LONG DispDate(VOID);
 VOID Editor(LONG);

 VOID PopVarStack(VOID);
 WORD EnterVar(UBYTE *,WORD,UBYTE *,LONG,WORD);
 UBYTE *FindVar(UBYTE *,WORD,WORD *);
 LONG DoMacro(UBYTE *,WORD);
 MACRO FindMacro(UBYTE *,WORD);
 LONG CountMacro(LINE,LINE);
 UBYTE *CopyToMacro(LINE,LINE,UBYTE *);
 LONG MakeMacro(UBYTE *,WORD);
 WORD WordTest(LINE,BYTE *);
 WORD DoAll(MACRO);
 LINE DoIfWhile(LINE,WORD);
 UBYTE *FindName(UBYTE *,WORD,UBYTE **,WORD *);
 LONG DoCommand(LINE,LINE);
 WORD NumVal(UBYTE *,LONG,LONG *);
 WORD NumToStr(LONG);
 WORD LogVal(UBYTE *,LONG);
 VARIABLE *Operation(VARIABLE *,VARIABLE *);
 LONG Compute(LINE);
 WORD EvalExpression(LINE,WORD,VARIABLE *,VARIABLE *);
 LONG DoCondition(LINE,WORD);
 WORD FindLabel(MACRO,LINE,WORD);
 LONG Xmacro(MACRO);
 LONG DeleteMacro(UBYTE *,WORD);
 MACRO LoadMacro(WORD,UBYTE *,WORD);
 MACRO CreaMacro(BUFFER,UBYTE *,WORD);
 LONG ShowVariable(UBYTE *,WORD);
 LONG GetFirst(UBYTE *,WORD);
 LONG Pause(UBYTE *,WORD);
 VOID setmacdir(UBYTE *);
 LONG GetString(VOID);
 WORD CheckParms(MACRO,WORD);

 LONG KeyDefRead(UBYTE *,WORD);
 LONG xtern(UBYTE *,WORD);
 LONG exprog(VOID);
 LONG InsertOne(WORD);
 LONG escape(VOID);
 LONG newline(WORD);
 LONG backspace(VOID);
 LONG delete(VOID);
 VOID iniCommand(VOID);
 LONG CountC(UBYTE *,WORD);
 LONG Istring(UBYTE *,WORD);

 UBYTE *GetKBshift(VOID);

 WORD Concat1(LINE,WORD,LINE,LINE);
 WORD Concat2(LINE,LINE,WORD,LINE);
 WORD Concat3(LINE,WORD,LINE,WORD,LINE);
 WORD Cpline(LINE,WORD,WORD,LINE);
 WORD CopyToEnd(LINE,WORD,LINE);
 WORD InsText(LINE,WORD,LINE,LINE);
 LINE Dsline(LINE,WORD,WORD);
 VOID ClearYank(VOID);
 VOID ClearPaste(VOID);
 LONG kcopy2(VOID);
 LONG kcopy(VOID);
 LONG copy(VOID);
 LONG kcut(VOID);
 LONG cut(VOID);
 LONG kpaste(VOID);
 LONG paste(VOID);

 LINE splitlin(LINE,WORD);
 WORD AskVeto(UBYTE *);
 WORD DoMatch(LINE,UBYTE *,WORD,LINE *,WORD *);
 WORD DetBounds(WORD);
 LONG forward(VOID);
 LONG backward(VOID);
 VOID FindPosc(VOID);
 LONG Sonly(VOID);
 LONG SandR(VOID);
 VOID DoOptions(UBYTE *,WORD);
 LONG Search(UBYTE *,WORD);
 LONG haakje(VOID);
 LONG wordoo(VOID);
 LONG wordol(VOID);
 LONG Freplace(UBYTE *,WORD);
 VOID MakeTable(VOID);
 VOID IniSearch(VOID);
 LONG RegExp(UBYTE *,WORD,WORD);
 LINE RegDetBounds(WORD *,LINE *);
 LINE RegGetEnd(UBYTE **);

 LONG Lsearch(VOID);

 LONG wordnx(VOID);
 LONG wordpr(VOID);
 LONG wordel(VOID);
 LONG wordlf(VOID);
 LONG WrapWord(LINE,WORD);
 LONG rewrap(VOID);
 LONG AutoIndent(UBYTE *,WORD);
 LONG WordWrap(UBYTE *,WORD);
 WORD NotInParagraph(LINE);
 #ifdef WITHOUTLINE
  LONG outline(VOID);
 #endif

 LONG Order(UBYTE *,WORD);
 WORD Sort(LINE,LINE);
 WORD Compare(LINE,LINE);
 VOID SplitMerge(LINE *,LONG);

 WORD WhatScreen(WORD *,WORD *,WORD *,WORD *);

 LONG TestMouse(WORD);
 LONG PcsMouse(LONG);
 #ifdef WITHMOUSE
  VOID MouseIni(VOID);
  VOID EndMouse(VOID);
  /* VOID xy_mouse(WORD,WORD); */
  /* VOID OurMouse(UBYTE *,WORD); */
  VOID InstalCleanup(VOID);
  /* VOID CleanTerm(VOID); */
  VOID Show_Mouse(VOID);
  VOID HideMouse(VOID);
  LONG ReadSelect(VOID);
  LONG WriteSelect(VOID);
 #endif

 VOID DumpMem(VOID);
 VOID DoLong(LONG,WORD);
 LONG TestMem(VOID);
 WORD DoAfold(LINE,LINE);
 WORD DoBfold(LINE,LINE);

 #ifdef UNIX
  WORD Filetype(BYTE *);
  VOID PSdraw(VOID);
 #endif

#else

 extern LONG l_arrow(),sl_arr(),cl_arr(),shctle(),r_arrow(),sr_arr(),
		cr_arr(),shctri(),u_arrow(),su_arr(),ctrup(),d_arrow(),
		sd_arr(),ctrdo(),clrhom(),sh_hom(),ctlhom(),cthome(),
		gotoln(),chline(),MoveColumn(),MoveLast(),MovePlus(),MoveMin(),
		MovePoint(),MoveCursor(),InsertOne(),escape(),newline(),backspace(),
		delete(),WrapWord(),rewrap(),Cinsert(),Linsert(),InNorm(),
		CRnorm(),Tdelete(),Cdelete(),Bdelete(),Ldelete(),setmrk(),
		remomk(),gotomk(),exgmrk(),twidle(),deleol(),wordnx(),
		wordpr(),wordel(),wordlf(),insert(),append(),puth(),
		AutoIndent(),WordWrap(),MyOpen(),MyCreate(),Exec(),ReadFile(),
		ReadC(),readkey(),WriteLine(),WriteFile(),WriteC(),writekey(),
		Print(),klaar(),save(),savek(),SaveS(),Quit(),RecallMessage(),
		ErrorMes(),NextKey(),CloseCFold(),OpenCFold(),OpenAll(),
		CloseAll(),OpenNum(),CloseNumber(),OpenName(),CloseName(),COpenFold(),
		CCloseFold(),GetTicks(),xtern(),exprog(),FilToDef(),DefToFil(),
		Dfaults(),help(),StartLearn(),pathnam(),set_sbr(),set_raw(),
		tog_tab(),tog_hexset(),set_ynk(),set_auu(),set_ioi(),set_ioo(),set_prn(),
		set_aua(),set_sns(),set_sdr(),togcol(),set_snn(),setview(),
		setback(),unsetv(),version(),GotoBuffer(),alt1(),alt2(),
		alt3(),alt4(),alt5(),alt6(),alt7(),alt8(),lfeed(),creturn(),
		alt9(),alt0(),control(),ctrl1(),ctrl2(),ctrl3(),
		ctrl4(),ctrl5(),ctrl6(),ctrl7(),ctrl8(),ctrl9(),
		ctrl0(),unknown(),ScreenU(),CountBytes(),Where(),rawkey(),HorStep(),
		FindBuffer(),Input(),Alttw(),CtrlC(),FunKey(),Command(),OneKeyUp(),
		repeat(),Jump(),GetMacro(),GetAkey(),Syntax(),intkey(),
		NoFold(),NoMark(),Convert(),upper(),lower(),flip(),FlipLines(),
		Lsearch(),forward(),backward(),Sonly(),SandR(),
		Search(),haakje(),wordoo(),wordol(),Freplace(),
		FindNum(),CountLines(),NumLines(),Trim(),Tabbing(),DeTab(),
		ItoL(),DiskFree(),Tab(),empty(),AddUndo(),CountC(),Istring(),
		undo(),kcopy2(),kcopy(),copy(),kcut(),cut(),
		kpaste(),paste(),repeat2(),GotoTag(),PutTag(),ForCommand(),
		Tag1(),Tag2(),Tag3(),Tag4(),Tag5(),Tag6(),Tag7(),Tag8(),Tag9(),Tag0(),
		KeyGet(),KeyChar(),KeyRead(),FlipSet(),FlipScreen(),KeyDefRead(),
		SetSome(),Order(),AutoClose(),DispDate(),RegExp(),Pause(),
		tog_io(),tog_ns(),tog_au(),tog_ua(),deadzone(),garb(),Yank(),Get();
 extern WORD Distend(),NotInParagraph(),MyRename(),Sfirst(),ChMod(),
		MyRemove(),backup(),SetKey(),GetLearn(),maskcmp(),KeyStart(),
		StartMem(),Garbage(),FillBlock(),DoAfold(),DoBfold(),GetRange(),
		ExpandLine(),Distance(),MesAsk(),AskVeto(),DoMatch(),DetBounds(),
		FindChar(),FindColumn(),CopyLine(),Range(),strNcmp(),strNIcmp(),
		StrCmp(),strcmp1(),ReadHex(),DetWor(),Concat1(),Concat2(),Concat3(),
		Cpline(),CopyToEnd(),InsText(),KeyPoll(),IsAvailable(),
		IsBrake(),IsCaps(),StrCIcmp(),GetOption(),Sort(),Compare(),
		ColToChar(),FindFold(),MakeDate(),DoDate(),CheckParms();
 extern WORD WhatScreen();
 extern VOID IniKbuffer(),InStall(),ReStall();
 extern VOID ToEOL(),UpOne(),DownOne(),iniCommand(),ScrollUp(),ScrollDown(),
		PullUp(),DetMark(),ClearHexMode(),KeyFlush(),RemoveFold(),KeyEnd(),
		SetScreen(),ResetScreen(),givename(),inibuf(),namfil(),LearnEnter(),
		QuitReplay(),LineDisplay(),TextCursor(),StrConv(),
		MakeTable(),Strcpy(),strNcpy(),LtoA(),PutOut(),
		iniundo(),Discard(),ClearUndo(),CopSet(),ExchSet(),BufUnd(),
		ClearPaste(),CursorOn(),CursorOff(),DumpMem(),DoLong(),TestMem(),
		CleanBlock(),MakeRMO(),MoveLine(),LineDealloc(),DetMarkColumn(),
		DropPart(),Display(),BufToDef(),DefToBuf(),SetDef(),setdefdir(),
		Editor(),KeyIni(),DefineWindow(),RedefScroll(),ShowDef(),MesNum(),
		MesNumS(),MesCon(),MesOut(),MessageForce(),MessageCommand(),
		MessageUpdate(),MessageRedo(),IniSearch(),Initialize(),UpdateAll(),
		MyWait(),SetHelp(),SplitMerge(),DoOptions(),MesMem(),FindPosc(),
		MesPrint(),MesIll(),PSaddwindow(),PSdelwindow(),DoDisplay(),
		UpdateDisplay(),ForceDisplay();
 extern VOID PSbegin(),PSend(),PSclreol(),PSclear(),PScolor(),PSsynch();
 extern VOID PSwrite(),PSsetcur(),PSshowcur(),PSbeep(),PSscroll(),PSrectclear();
 extern UBYTE *TakeScreen(),*ReadNum();
 #ifdef OWNENV
  BYTE *mygetenv();
 #endif
 extern SPECS *SpecKey();

 extern LINE InsertBuffer(),OpenFold(),FindStart(),FindEnd(),RegDetBounds(),
		LineAlloc(),splitlin(),FindLine(),Dsline(),RegGetEnd();

 extern FOLDLINE FoldAlloc(),CloseFold();

 extern UBYTE *GetKBshift();

 #ifdef WITHMOUSE
  extern VOID MouseIni(),EndMouse(),Show_Mouse(),HideMouse(),CursorBlink();
  extern LONG TestMouse(),PcsMouse(),ReadSelect(),WriteSelect();
 #endif

 LINE DoIfWhile();
 LONG Compute(),CountMacro(),DeleteMacro(),DoCommand(),ShowVariable();
 LONG DoCondition(),DoMacro(),MakeMacro(),Xmacro(),GetFirst();
 MACRO FindMacro(),LoadMacro(),CreaMacro();
 UBYTE *CopyToMacro(),*FindName(),*FindVar();
 VARIABLE *Operation();
 VOID PopVarStack(),setmacdir();
 WORD EnterVar(),EvalExpression(),FindLabel(),LogVal();
 WORD NumToStr(),NumVal(),WordTest(),DoAll();
 #ifdef LINUX
  VOID PSdraw();
 #endif

 WORD Excute(),PrintOne();
 
 VOID MesnOut();
 LONG GetString();
 #ifdef UNIX
  WORD Filetype();
  int read(),write(),close();
 #endif
#endif

#ifdef WITHOUTLINE
LONG outline();
#endif

/*
  	#] Functions :
*/

extern FILE *fdebug;
