/*
  	#[ Include file :
*/
#include "stedi.h"
/*
  	#] Include file : 
  	#[ Undo :
 		#[ Initialize the Undo buffer :
*/

VOID
iniundo(VOID)
{
	BUFFER b = undoB;
	b->first = b->last = b->current = b->mark = (LINE)b;
	b->undo = 0;
	undone = 0;
}

/*
 		#] Initialize the Undo buffer : 
 		#[ Clear the Undo buffer :
*/

VOID
Discard(LINE l)
{
	LINE stop;
	if ( l->used < 0 ) {
		stop = ((FOLDLINE)l)->first->prev;
		do { Discard(stop->next); } while ( stop->next != stop );
	}
	LineDealloc(l);
}

VOID
ClearUndo(VOID)
{
	BUFFER b = undoB;
	while ( b->first != (LINE)b ) Discard(b->first);
	iniundo();
}

/*
 		#] Clear the Undo buffer : 
 		#[ Copy settings :
*/

VOID
CopSet(WORD from,WORD to,WORD type)
{
	BUFFER b;
	BUFFER u;
	WORD i;
	WORD *van, *naar;
	b = textbuffer[from];
	u = textbuffer[to];
	if ( type == NONAMES ) {
		van = &(b->MarkColumn); naar = &(u->MarkColumn);
		i = (WORD)(((WORD *)b->name) - van);
	}
	else {
		van = &(b->bufnum); naar = &(u->bufnum);
		i = (WORD)((WORD *)(b+1) - van);
	}
	while ( --i >= 0 ) *naar++  = *van++;
	u->undo   = type;
}

/*
 		#] Copy settings : 
 		#[ Exchange settings with Undo :
*/

VOID
ExchSet(WORD num)
{
	BUFFER b;
	WORD i, x, *v1, *v2;
	b = textbuffer[num];
	v1 = &(undoB->bufnum); v2 = &(b->bufnum);
	i = (WORD)(((WORD *)(b+1)) - v2);
	while ( --i >= 0 ) {
		x = *v1; *v1++ = *v2; *v2++ = x;
	}
}

/*
 		#] Exchange settings with Undo : 
 		#[ Cut buffer to Undo :
*/

VOID
BufUnd(WORD bufnum)
{
	BUFFER b;
	BUFFER u = undoB;
	ClearUndo();
	b = textbuffer[bufnum];
	CopSet(bufnum,UNDOCODE,UNDOBUFFER);
	if ( b->first != (LINE)b ) {
		u->first = b->first;   u->last = b->last;
		u->first->prev = u->last->next = (LINE)u;
	}
	else { u->first = u->last = (LINE)u; }
	if ( b->current != (LINE)b ) u->current = b->current;
	else u->current = (LINE)u;
	if ( b->mark != (LINE)b ) u->mark = b->mark;
	else u->mark = (LINE)u;
	inibuf(bufnum);					/* Delete the old links */
}

/*
 		#] Cut buffer to Undo : 
 		#[ ClearBuffer() :
*/

LONG
empty(VOID)
{
	BUFFER b = cbuf;
	if ( b->dirtyflag > 0 ) {
		MesnOut(82);
		if ( MesAsk(100,INCOMMAND) <= 0 ) return(-1L);
		/* "Buffer unsaved. Empty anyway? (Y/N) " */
	}
	BufUnd(b->bufnum);
	ForceDisplay();
	return(0L);
}

/*
 		#] ClearBuffer() : 
 		#[ AddUndo :

		par = UNDOLINE		A full line should be added. Remove from original!
		par = UNDOPART		Part of a line should be added. Test whether it
							fits in with what we have
		par = UNDOEOL		The end of line after l is deleted.

		The ordinary positions of the current line and the mark
		are now taken by the lines in front and after what has
		been put in the undo buffer.

*/

LONG
AddUndo(LINE l,WORD par)
{
	BUFFER b = cbuf, u = undoB;
	WORD par2, i;
	LINE l2;
	if ( l == (LINE)b ) return(0L);
	if ( par == UNDOEOL ) par2 = UNDOPART;
	else				  par2 = par;
	if ( ( u->undo != 0 ) && ( u->undo != UNDOBUFFER ) && !undone ) {
		if ( ( par == UNDOLINE ) && ( u->undo == UNDOLINE ) ) {
			if ( l == u->current ) {		/* add to top */
				u->current = l->prev;
				l->prev->next = l->next;	/* Remove from file */
				l->next->prev = l->prev;
				l->next = u->first;
				l->prev = (LINE)u;
				u->first = l->next->prev = l;
				return(0L);
			}
			else if ( l == u->mark ) {		/* add to bottom */
				u->mark = l->next;
				l->prev->next = l->next;	/* Remove from file */
				l->next->prev = l->prev;
				l->prev = u->last;
				l->next = (LINE)u;
				u->last = l->prev->next = l;
				return(0L);
			}
		}
		else if ( ( par2 == UNDOPART ) && ( u->undo == UNDOPART ) ) {
			if ( l == u->current ) {
				if ( CopyLine(l,(LINE)u) ) { return(NOMEMORYL); }
				u->current = l->prev;
				return(0L);
			}
			else if ( ( l->next == u->current ) && ( par == UNDOEOL ) ) {
				if ( CopyLine(l,(LINE)u) ) { return(NOMEMORYL); }
				u->current = l->prev;
				l = l->next;
				if ( CopyLine(l,u->first) ) {
					LineDealloc(u->first);
					u->current = l;
					return(NOMEMORYL);
				}
				return(0L);
			}
			else if ( l == u->mark ) {
				if ( CopyLine(l,u->last) ) { return(NOMEMORYL); }
				if ( par == UNDOEOL ) {
					l = l->next;
					if ( CopyLine(l,u->last) ) {
						LineDealloc(u->last);
						return(NOMEMORYL);
					}
				}
				u->mark = l->next;
				return(0L);
			}
			else if ( ( l->next == u->mark ) && ( par == UNDOEOL ) ) {
				if ( CopyLine(l->next,u->last) ) { return(NOMEMORYL); }
				u->mark = l->next->next;
				return(0L);
			}
			else {
				i = 0;
				l2 = u->first;
				while ( l2 != (LINE)u ) { i++; l2 = l2->next; }
				if ( par == UNDOEOL ) i--;
				l2 = l->prev;
				while ( --i >= 0 ) {
					if ( l2 == (LINE)b ) i = 0;
					if ( l2 == u->current ) return(0L);
					l2 = l2->prev;
				}
			}
		}
	}
/*
 *	Clean up the undo buffer first.
 */
	ClearUndo();
	CopSet(b->bufnum,UNDOCODE,par2);
	if ( par == UNDOLINE ) {
		u->first = u->last = l;
		u->current = l->prev;
		u->mark = l->next;
		l->prev->next = l->next;
		l->next->prev = l->prev;
		l->next = l->prev = (LINE)u;
	}
	else if ( par == UNDOPART ) {
		if ( CopyLine(l,(LINE)u) ) { u->undo = 0; return(NOMEMORYL); }
		u->current = l->prev;
		u->mark = l->next;
	}
	else if ( par == UNDOEOL ) {
		if ( CopyLine(l,(LINE)u) ) { u->undo = 0; return(NOMEMORYL); }
		u->current = l->prev;
		l = l->next;
		if ( CopyLine(l,u->first) ) {
			LineDealloc(u->first);
			u->current = (LINE)u;
			u->undo = 0;
			return(NOMEMORYL);
		}
		u->mark = l->next;
	}
	return(0L);
}

/*
 		#] AddUndo : 
 		#[ The undo itself :
*/

LONG
undo(VOID)
{
	BUFFER b = 0;
	BUFFER u = undoB;
	LINE l2;
	WORD i;
	if ( commandmode ) return(0);
	switch ( u->undo ) {

		case 0 : return(0L);

		case UNDOBUFFER :

				b = textbuffer[u->bufnum];
				CopSet(UNDOCODE,u->bufnum,0);
				b->first = u->first;
				b->last = u->last;
				u->first->prev = u->last->next = (LINE)b;
				if ( u->mark != (LINE)u ) b->mark = u->mark;
				else b->mark = (LINE)b;
				iniundo();
				break;

		case UNDOLINE :

				b = textbuffer[u->bufnum];
				CopSet(UNDOCODE,u->bufnum,0);
				u->current->next = u->first;
				u->mark->prev = u->last;
				u->first->prev = u->current;
				u->last->next = u->mark;
				iniundo();
				break;

		case UNDOPART :			/* Switch contents */

				undone = 1 - undone;
				b = textbuffer[u->bufnum];
				ExchSet(u->bufnum);

				l2 = u->first;
				u->current->next->prev = (LINE)u;		/* Switch the front */
				l2->prev = u->current;
				u->first = u->current->next;
				u->current->next = l2;

				l2 = u->last;
				u->mark->prev->next = (LINE)u;			/* Switch the end */
				l2->next = u->mark;
				u->last = u->mark->prev;
				u->mark->prev = l2;

				if ( b->mark != (LINE)b ) {		/* Try to reconstruct the mark */
					l2 = u->first;
					i = 1;
					while ( l2 != (LINE)u ) {
						if ( l2 == b->mark ) {
							l2 = u->current;
							while ( --i >= 0 ) l2 = l2->next;
							b->mark = l2;
							l2 = (LINE)u;
						}
						else {
							i++;
							l2 = l2->next;
						}
					}
				}
				break;
	}
	b->current = FindLine(b->newline);
	return(GotoBuffer(b->bufnum));
}

/*
 		#] The undo itself : 
  	#] Undo : 
*/

